@@ -6,6 +6,7 @@ module SModel (module SModel,
                module SModel.Likelihood) where 
 import Probability
 import Bio.Alphabet
+import Bio.Sequence
 import Tree
 import Parameters
 
@@ -54,27 +55,29 @@ rate (MixtureModel d) = average [(p,rate m) | (p,m) <- d]
 scale x (ReversibleMarkov a s q pi l t r) = ReversibleMarkov a s q pi l (x*t) (x*r)
 scale x (MixtureModel dist              ) = MixtureModel [(p, scale x m) | (p, m) <- dist]
 
-rescale q r = scale (r/rate q) q
+rescale r q = scale (r/rate q) q
 
 mixMM fs ms = MixtureModel $ mix fs [m | MixtureModel m <- ms]
 scale_MMs rs ms = [scale r m | (r,m) <- zip' rs ms]
 
 -- For mixtures like mixture([hky85,tn93,gtr]), we probably need to mix on the Matrix level, to avoid shared scaling.
+mixture ms fs = mixMM fs ms
+-- Note that this scales the models BY rs instead of TO rs.
 scaled_mixture ms rs fs = mixMM fs (scale_MMs rs ms)
 
 parameter_mixture :: (a -> MixtureModel b) -> [a] -> MixtureModel b
-parameter_mixture model_fn values = MixtureModel [ (f*p, m) |(p,x) <- values, let MixtureModel dist = model_fn x, (f,m) <- dist]
+parameter_mixture values model_fn = MixtureModel [ (f*p, m) |(p,x) <- values, let MixtureModel dist = model_fn x, (f,m) <- dist]
 
 parameter_mixture_unit :: (a -> ReversibleMarkov b) -> [a] -> MixtureModel b
-parameter_mixture_unit model_fn values = parameter_mixture (unit_mixture . model_fn) values
+parameter_mixture_unit values model_fn = parameter_mixture values (unit_mixture . model_fn)
 
-rate_mixture m d = parameter_mixture (\x->scale x m) d
+rate_mixture m d = parameter_mixture d (\x->scale x m)
 
 average_frequency (MixtureModel ms) = list_from_vector $ builtin_average_frequency $ weighted_frequency_matrix $ MixtureModel ms
 
 extend_mixture (MixtureModel ms) (p,x) = MixtureModel $ mix [p, 1.0-p] [certainly x, ms]
 
-plus_inv mm p_inv = extend_mixture mm (p_inv, scale 0.0 $ f81 pi a)
+plus_inv p_inv mm = extend_mixture mm (p_inv, scale 0.0 $ f81 pi a)
     where a  = getAlphabet mm
           pi = average_frequency mm
 
@@ -125,36 +128,36 @@ m8a_test_omega_dist mu gamma n_bins posP posW _ = m8_omega_dist mu gamma n_bins
 
 --  w1 <- uniform 0.0 1.0
 --  [f1, f2] <- dirichlet' 2 1.0
-m1a model_func w1 f1 = parameter_mixture_unit model_func (m1a_omega_dist f1 w1)
+m1a w1 f1 model_func = parameter_mixture_unit (m1a_omega_dist f1 w1) model_func
 
-m2a model_func w1 f1 posP posW = parameter_mixture_unit model_func (m2a_omega_dist f1 w1 posP posW)
+m2a w1 f1 posP posW model_func = parameter_mixture_unit (m2a_omega_dist f1 w1 posP posW) model_func
 
-m2a_test model_func w1 f1 posP posW posSelection = parameter_mixture_unit model_func (m2a_test_omega_dist f1 w1 posP posW posSelection)
+m2a_test w1 f1 posP posW posSelection model_func = parameter_mixture_unit (m2a_test_omega_dist f1 w1 posP posW posSelection) model_func
 
-m3 model_func ps omegas = parameter_mixture_unit model_func (m3_omega_dist ps omegas)
+m3 ps omegas model_func = parameter_mixture_unit (m3_omega_dist ps omegas) model_func
 
-m3_test model_func ps omegas posP posW posSelection = parameter_mixture_unit model_func (m3_test_omega_dist ps omegas posP posW posSelection)
+m3_test ps omegas posP posW posSelection model_func = parameter_mixture_unit (m3_test_omega_dist ps omegas posP posW posSelection) model_func
 
-m7 model_func mu gamma n_bins =  parameter_mixture_unit model_func (m7_omega_dist mu gamma n_bins)
+m7 mu gamma n_bins model_func =  parameter_mixture_unit (m7_omega_dist mu gamma n_bins) model_func
 
-m8 model_func mu gamma n_bins posP posW = parameter_mixture_unit model_func (m8_omega_dist mu gamma n_bins posP posW)
+m8 mu gamma n_bins posP posW model_func = parameter_mixture_unit (m8_omega_dist mu gamma n_bins posP posW) model_func
 
-m8a model_func mu gamma n_bins posP = parameter_mixture_unit model_func (m8a_omega_dist mu gamma n_bins posP)
+m8a mu gamma n_bins posP model_func = parameter_mixture_unit  (m8a_omega_dist mu gamma n_bins posP) model_func
 
-m8a_test model_func mu gamma n_bins posP posW posSelection = parameter_mixture_unit model_func (m8a_test_omega_dist mu gamma n_bins posP posW posSelection)
+m8a_test mu gamma n_bins posP posW posSelection model_func = parameter_mixture_unit (m8a_test_omega_dist mu gamma n_bins posP posW posSelection) model_func
 
 -- OK, so if I change this from [Mixture Omega] to Mixture [Omega] or Mixture (\Int -> Omega), how do I apply the function model_func to all the omegas?
-branch_site model_func fs ws posP posW branch_cats = MixtureModels branch_cats [bg_mixture,fg_mixture]
+branch_site fs ws posP posW branch_cats model_func = MixtureModels branch_cats [bg_mixture,fg_mixture]
 -- background omega distribution -- where the last omega is 1.0 (neutral)
     where bg_dist = zip fs (ws ++ [1.0])
 -- accelerated omega distribution -- posW for all categories
           accel_dist = zip fs (repeat posW)
 -- background branches always use the background omega distribution              
-          bg_mixture = parameter_mixture_unit model_func (mix [1.0-posP, posP] [bg_dist, bg_dist])
+          bg_mixture = parameter_mixture_unit (mix [1.0-posP, posP] [bg_dist, bg_dist]) model_func
 -- foreground branches use the foreground omega distribution with probability posP
-          fg_mixture = parameter_mixture_unit model_func (mix [1.0-posP, posP] [bg_dist, accel_dist])
+          fg_mixture = parameter_mixture_unit (mix [1.0-posP, posP] [bg_dist, accel_dist]) model_func
 
-branch_site_test model_func fs ws posP posW posSelection = branch_site model_func fs ws posP posW'
+branch_site_test fs ws posP posW posSelection branch_cats model_func = branch_site fs ws posP posW' branch_cats model_func
     where posW' = if (posSelection == 1) then posW else 1.0
 
 mut_sel w' (ReversibleMarkov a smap q0 pi0 _ _ _) = reversible_markov a smap q pi where
@@ -211,6 +214,10 @@ modulated_markov models rates_between level_probs = reversible_markov a smap q p
     pi = modulated_markov_pi pis level_probs
     smap = modulated_markov_smap smaps
 
+markov_modulate_mixture nu (MixtureModel dist) = modulated_markov models rates_between level_probs where
+    (level_probs,models) = unzip dist
+    rates_between = generic_equ (length models) nu
+
 -- We need to rescale submodels to have substitution rate `1.0`.
 -- Otherwise class-switching rates are not relative to the substitution rate.
 
@@ -218,14 +225,14 @@ tuffley_steel_98_unscaled s01 s10 q = modulated_markov [scale 0.0 q, q] rates_be
     level_probs = [s10/total, s01/total] where total = s10 + s01
     rates_between = fromLists [[-s01,s01],[s10,-s10]]
 
-tuffley_steel_98 s01 s10 q = tuffley_steel_98_unscaled s01 s10 (rescale q 1.0)
+tuffley_steel_98 s01 s10 q = tuffley_steel_98_unscaled s01 s10 (rescale 1.0 q)
 
 huelsenbeck_02 s01 s10 model = MixtureModel [(p, tuffley_steel_98_unscaled s01 s10 q) | (p,q) <- dist] where
-    MixtureModel dist = rescale model 1.0
+    MixtureModel dist = rescale 1.0 model
 
 galtier_01_ssrv :: Double -> MixtureModel a -> ReversibleMarkov a
 galtier_01_ssrv nu model = modulated_markov models rates_between level_probs where
-    MixtureModel dist = rescale model 1.0
+    MixtureModel dist = rescale 1.0 model
     level_probs = map fst dist
     models = map snd dist
     n_levels = length dist
@@ -233,26 +240,26 @@ galtier_01_ssrv nu model = modulated_markov models rates_between level_probs whe
     rates_between = (generic_equ n_levels nu) %*% (plus_f_matrix $ list_to_vector level_probs)
 
 galtier_01 :: Double -> Double -> MixtureModel a -> MixtureModel a
-galtier_01 nu pi model = parameter_mixture_unit (\nu' -> galtier_01_ssrv nu' model) [(1.0-pi, 0.0), (pi, nu)]
+galtier_01 nu pi model = parameter_mixture_unit [(1.0-pi, 0.0), (pi, nu)] (\nu' -> galtier_01_ssrv nu' model)
 
 wssr07_ssrv :: Double -> Double -> Double -> MixtureModel a -> ReversibleMarkov a
 wssr07_ssrv s01 s10 nu model = tuffley_steel_98 s01 s10 $ galtier_01_ssrv nu model
 
 wssr07 :: Double -> Double -> Double -> Double -> MixtureModel a -> MixtureModel a
-wssr07 s01 s10 nu pi model = parameter_mixture_unit (\nu' -> wssr07_ssrv s01 s10 nu' model) [(1.0-pi, 0.0), (pi, nu)]
+wssr07 s01 s10 nu pi model = parameter_mixture_unit [(1.0-pi, 0.0), (pi, nu)] (\nu' -> wssr07_ssrv s01 s10 nu' model)
 
 gamma_rates_dist alpha = gamma alpha (1.0/alpha)
 
-gamma_rates base alpha n = rate_mixture_unif_bins base (gamma_rates_dist alpha) n
+gamma_rates alpha n base = rate_mixture_unif_bins base (gamma_rates_dist alpha) n
 
 log_normal_rates_dist sigmaOverMu = log_normal lmu lsigma where x = log(1.0+sigmaOverMu^2)
                                                                 lmu = -0.5*x
                                                                 lsigma = sqrt x
 
-log_normal_rates base sigmaOverMu n = rate_mixture_unif_bins base (log_normal_rates_dist sigmaOverMu) n
+log_normal_rates sigmaOverMu n base = rate_mixture_unif_bins base (log_normal_rates_dist sigmaOverMu) n
 
 --dp base rates fraction = rate_mixture base dist where dist = zip fraction rates
-free_rates base rates fractions = scaled_mixture (replicate (length fractions) base) rates fractions
+free_rates rates fractions base = scaled_mixture (replicate (length fractions) base) rates fractions
 
 transition_p_index smodel_on_tree = mkArray n_branches (list_to_vector . branch_transition_p smodel_on_tree) where tree = get_tree smodel_on_tree
                                                                                                                    n_branches = numBranches tree
@@ -362,7 +369,7 @@ componentFrequencies        (MixtureModels _ (m:ms))         i = componentFreque
 
 unit_mixture m = MixtureModel (certainly m)
 
-mmm m branch_cats = MixtureModels branch_cats [m]
+mmm branch_cats m = MixtureModels branch_cats [m]
 
 empirical a filename = builtin_empirical a (list_to_string filename)
 
@@ -371,13 +378,15 @@ lg_frequencies a = zip (letters a) (list_from_vector $ builtin_lg_frequencies a)
 
 -- FIXME: need polymorphism.
 --        This needs to be after weighted_frequency_matrix.
---        Because we have no polymorphism, wfm needs to be after MixtureModel and MixtureModels.
-subst_like_on_tree topology root as alphabet smodel ts scale seqs = substitution_likelihood topology root seqs' as alphabet ps f smap
-    where f = weighted_frequency_matrix smodel
-          ps = transition_p_index (SingleBranchLengthModel topology ds smodel)
+--        Because we have no polymorphism, wfm needs to be defined after MixtureModel and MixtureModels.
+subst_like_on_tree topology root as smodel ts scale seqs = substitution_likelihood topology root seqs' as alphabet ps f smap
+    where taxa = get_labels topology
+          f = weighted_frequency_matrix smodel
           ds = listArray' $ map (scale*) ts
-          seqs' = listArray' seqs
+          ps = transition_p_index (SingleBranchLengthModel topology ds smodel)
+          seqs' = listArray' $ map (sequence_to_indices alphabet) $ reorder_sequences taxa seqs
+          alphabet = getAlphabet smodel
           smap = get_smap smodel
 
-ctmc_on_tree topology root as alphabet smodel ts scale =
-    Distribution (\seqs -> [subst_like_on_tree topology root as alphabet smodel ts scale seqs]) (no_quantile "ctmc_on_tree") () ()
+ctmc_on_tree topology root as smodel ts scale =
+    Distribution (\seqs -> [subst_like_on_tree topology root as smodel ts scale seqs]) (no_quantile "ctmc_on_tree") () ()
