@@ -348,7 +348,7 @@ std::string generate_print_program(const model_t& print, const expression_ref& a
     }
 
     if (print.code.is_action())
-        program_file<<"  result <- run_lazy (print_model alphabet)\n";
+        program_file<<"  result <- run_lazy ("<<E<<")\n";
     else
         program_file<<"  let result = "<<E<<"\n";
 
@@ -446,12 +446,11 @@ int main(int argc,char* argv[])
     owned_ptr<Model> M; // Declare early so we can reference from catch block.
 
     try {
-
-#if defined(HAVE_FEENABLEEXCEPT) && !defined(NDEBUG)
+// This should probably be a run-time switch.
+#if defined(HAVE_FEENABLEEXCEPT) && defined(DEBUG_FPE)
         feenableexcept(FE_DIVBYZERO|FE_OVERFLOW|FE_INVALID);
-        //    feclearexcept(FE_DIVBYZERO|FE_OVERFLOW|FE_INVALID);
 #endif
-#if defined(HAVE_CLEAREXCEPT) && defined(NDEBUG)
+#if defined(HAVE_CLEAREXCEPT) && !defined(DEBUG_FPE)
         feclearexcept(FE_DIVBYZERO|FE_OVERFLOW|FE_INVALID);
 #endif
         fp_scale::initialize();
