@@ -1368,10 +1368,10 @@ using boost::dynamic_bitset;
 
 namespace substitution {
     Likelihood_Cache_Branch*
-    peel_leaf_branch(const EVector& sequence, const EVector& counts, const alphabet& a, const EVector& transition_P, const EVector& smap);
+    peel_leaf_branch(const EVector& sequence, const alphabet& a, const EVector& transition_P, const EVector& smap);
 
     Likelihood_Cache_Branch*
-    peel_leaf_branch_SEV(const EVector& sequence, const alphabet& a, const EVector& transition_P, const dynamic_bitset<>& mask);
+    peel_leaf_branch_SEV(const EVector& sequence, const alphabet& a, const EVector& transition_P, const dynamic_bitset<>& mask, const EVector& smap);
 }
 
 extern "C" closure builtin_function_peel_leaf_branch(OperationArgs& Args)
@@ -1380,9 +1380,8 @@ extern "C" closure builtin_function_peel_leaf_branch(OperationArgs& Args)
     auto arg1 = Args.evaluate(1);
     auto arg2 = Args.evaluate(2);
     auto arg3 = Args.evaluate(3);
-    auto arg4 = Args.evaluate(4);
 
-    return substitution::peel_leaf_branch(arg0.as_<EVector>(), arg1.as_<EVector>(), *arg2.as_<Alphabet>(), arg3.as_<EVector>(), arg4.as_<EVector>());
+    return substitution::peel_leaf_branch(arg0.as_<EVector>(), *arg1.as_<Alphabet>(), arg2.as_<EVector>(), arg3.as_<EVector>());
 }
 
 
@@ -1392,8 +1391,9 @@ extern "C" closure builtin_function_peel_leaf_branch_SEV(OperationArgs& Args)
     auto arg1 = Args.evaluate(1);
     auto arg2 = Args.evaluate(2);
     auto arg3 = Args.evaluate(3);
+    auto arg4 = Args.evaluate(4);
 
-    return substitution::peel_leaf_branch_SEV(arg0.as_<EVector>(), *arg1.as_<Alphabet>(), arg2.as_<EVector>(), arg3.as_<Box<dynamic_bitset<>>>());
+    return substitution::peel_leaf_branch_SEV(arg0.as_<EVector>(), *arg1.as_<Alphabet>(), arg2.as_<EVector>(), arg3.as_<Box<dynamic_bitset<>>>(), arg4.as_<EVector>());
 }
 
 
@@ -1573,14 +1573,12 @@ extern "C" closure builtin_function_sample_root_sequence_SEV(OperationArgs& Args
     auto arg2 = Args.evaluate(2);
     auto arg3 = Args.evaluate(3);
     auto arg4 = Args.evaluate(4);
-    auto arg5 = Args.evaluate(5);
-    auto arg6 = Args.evaluate(6);
 
     return substitution::sample_root_sequence_SEV(arg0.as_<Likelihood_Cache_Branch>(),
-                                              arg1.as_<Likelihood_Cache_Branch>(),
-                                              arg2.as_<Likelihood_Cache_Branch>(),
-                                              arg3.as_<Box<Matrix>>(),
-                                              arg3.as_<EVector>());
+                                                  arg1.as_<Likelihood_Cache_Branch>(),
+                                                  arg2.as_<Likelihood_Cache_Branch>(),
+                                                  arg3.as_<Box<Matrix>>(),
+                                                  arg4.as_<EVector>());
 }
 
 extern "C" closure builtin_function_sample_internal_node_sequence(OperationArgs& Args)
@@ -1816,8 +1814,133 @@ extern "C" closure builtin_function_peel_likelihood_2(OperationArgs& Args)
 	{
 	    int l = seq2[j++].as_int();
 	    Pr *= letter_frequency(l, alpha, F, LF);
-
 	}
     }
     return {Pr};
 }
+
+extern "C" closure builtin_function_peel_likelihood_1_SEV(OperationArgs& Args)
+{
+    auto arg0 = Args.evaluate(0);
+    auto arg1 = Args.evaluate(1);
+    auto arg2 = Args.evaluate(2);
+    auto arg3 = Args.evaluate(3);
+
+    const auto& A        = arg0.as_<Box<alignment>>();
+    const alphabet& a    = *arg1.as_<Alphabet>();
+    const auto& WF       = arg2.as_<Box<Matrix>>();
+    const auto& counts   = arg3.as_<EVector>();
+
+    // Make frequency-vector AND log(frequency)-vector
+    vector<double> F(a.size(),0);
+    vector<log_double_t> LF(a.size());
+    for(int l=0;l<F.size();l++)
+    {
+	for(int m=0;m<WF.size1();m++)
+	    F[l] += WF(m,l);
+	LF[l] = F[l];
+    }
+
+    log_double_t Pr = 1;
+    for(int i=0;i<A.length();i++)
+    {
+        int l = A(i,0);
+        log_double_t p = letter_frequency(l, a, F, LF);
+        int count = counts[i].as_int();
+	Pr *= pow(p,count);
+    }
+
+    return {Pr};
+}
+
+extern "C" closure builtin_function_peel_likelihood_2_SEV(OperationArgs& Args)
+{
+    auto arg0 = Args.evaluate(0);
+    auto arg1 = Args.evaluate(1);
+    auto arg2 = Args.evaluate(2);
+    auto arg3 = Args.evaluate(3);
+    auto arg4 = Args.evaluate(4);
+
+    const auto& A         = arg0.as_<Box<alignment>>();
+    const alphabet& alpha = *arg1.as_<Alphabet>();
+    const auto& P         = arg2.as_<EVector>();
+    const auto& WF        = arg3.as_<Box<Matrix>>();
+    const auto& counts    = arg4.as_<EVector>();
+
+    assert(A.n_sequences() == 2);
+
+    // Make frequency-vector AND log(frequency)-vector
+    vector<double> F(alpha.size(),0);
+    vector<log_double_t> LF(alpha.size());
+    for(int l=0;l<F.size();l++)
+    {
+	for(int m=0;m<WF.size1();m++)
+	    F[l] += WF(m,l);
+	LF[l] = F[l];
+    }
+
+    log_double_t Pr = 1;
+
+    for(int x=0;x<A.length();x++)
+    {
+        int l1 = A(x,0);
+        int l2 = A(x,1);
+
+        if (l1 < 0 and l2 < 0) continue;
+
+        double p = 0;
+
+	if (l1 >= 0 and l2 >= 0)
+	{
+	    if (alpha.is_letter(l1))
+	    {
+		if (alpha.is_letter(l2))
+		{
+		    for(int m=0;m<WF.size1();m++)
+			p += WF(m,l1) * P[m].as_<Box<Matrix>>()(l1,l2);
+		}
+		else if (alpha.is_letter_class(l2))
+		{
+		    const auto & fmask = alpha.letter_fmask(l2);
+		    for(int m=0;m<WF.size1();m++)
+			for(int j=0; j<alpha.size(); j++)
+			    p += WF(m,l1) * P[m].as_<Box<Matrix>>()(l1,j) * fmask[j];
+		}
+                else
+                    std::abort();
+	    }
+	    else if (alpha.is_letter_class(l1))
+	    {
+		if (alpha.is_letter(l2))
+		{
+		    const auto & fmask = alpha.letter_fmask(l1); 
+		    for(int m=0;m<WF.size1();m++)
+			for(int j=0; j<alpha.size(); j++)
+			    p += WF(m,l2) * P[m].as_<Box<Matrix>>()(l2,j) * fmask[j];
+		}
+		else if (alpha.is_letter_class(l2))
+		{
+		    const auto & mask1 = alpha.letter_mask(l1);
+		    const auto & fmask2 = alpha.letter_fmask(l2);
+		    for(int m=0;m<WF.size1();m++)
+			for(int j=0; j<alpha.size(); j++)
+			    if (mask1.test(j))
+				for(int k=0; k<alpha.size(); k++)
+				    p += WF(m,j) * P[m].as_<Box<Matrix>>()(j,k) * fmask2[k];
+		}
+		else
+                    std::abort();
+	    }
+	}
+	else if (l1 >= 0)
+            p = letter_frequency(l1, alpha, F, LF);
+	else if (l2 >= 0)
+	    p = letter_frequency(l2, alpha, F, LF);
+        else
+            std::abort();
+
+        int count = counts[x].as_int();
+        Pr *= pow(log_double_t(p), count);
+    }
+    return {Pr};
+}
