@@ -7,7 +7,7 @@
 
 #include "expression.H"
 #include "computation/module.H"
-#include "util.H"
+#include "util/string/convert.H"
 #include "computation/operation.H"
 #include "computation/operations.H"
 #include "substitute.H"
@@ -16,6 +16,8 @@
 #include "var.H"
 #include "lambda.H"
 #include "bool.H"
+#include "modifiable.H"
+#include "reg_var.H"
 
 using std::vector;
 using std::string;
@@ -25,87 +27,6 @@ using std::multiset;
 
 using boost::dynamic_pointer_cast;
 
-// 1. factor out: list, tuple
-// Also try to simplify the apply_subst stuff in tuple & list files
-// 2. eliminate substitution.H
-// 3. Eliminate identifier in favor of var (==var)?
-// 4. Remove horrible (#symbol)*(#function) substitution in module.C
-
-bool parameter::operator==(const Object& o) const 
-{
-    const parameter* E = dynamic_cast<const parameter*>(&o);
-    if (not E) 
-	return false;
-
-    return parameter_name == E->parameter_name;
-}
-
-parameter::parameter(const std::string& s)
-    :parameter_name(s)
-{
-}
-
-/// 1. Hey, could we solve the problem of needing to rename dummies by doing capture-avoiding substitution?
-/// I think we could!
-///
-/// Suppose we have Lf.Lx.fx, and we apply it to Lx.x (the identity function), then we get
-///    (Lf.Lx.fx)(Lx.x) = (Lx.fx)[f := Lx.x] = Lx.(Lx.x)x.
-/// Then, if we apply this to y, we get
-///    (Lx.(Lx.x)x)y = (Lx.x)x[x := y] = (Lx.x)y 
-/// And
-///    (Lx.x)y = y;
-///
-/// 2. However, is sometimes still necessary to rename dummies.  This is true if E2 contains unbound dummies
-///    that are bound in E1.
-///
-///    For example, apply Lx.y to x, then we would get Lx.x, which is not allowed.
-///    Instead, we must "alpha-convert" Lx.y to Lz.y, and then apply Lz.y to x, leading to Lz.x .
-
-/// Literally E2 for D in E1. (e.g. don't rename variables in E2).  Throw an exception if D is a lambda-bound variable.
-
-void find_named_parameters(const expression_ref& E, std::set<string>& names)
-{
-    assert(E);
-    // If this is a parameter, then makes sure we've got its name.
-    if (E.is_a<parameter>())
-    {
-	auto& n = E.as_<parameter>();
-	assert(not E.size());
-	if (names.find(n.parameter_name) == names.end())
-	    names.insert(n.parameter_name);
-    }
-
-    // Check the sub-objects of this expression.
-    for(int i=0;i<E.size();i++)
-	find_named_parameters(E.sub()[i], names);
-}
-
-set<string> find_named_parameters(const expression_ref& e)
-{
-    set<string> names;
-    find_named_parameters(e, names);
-    return names;
-}
-
-set<string> find_named_parameters(const vector<expression_ref>& notes)
-{
-    set<string> names;
-    for(int i=0;i<notes.size();i++)
-	find_named_parameters(notes[i], names);
-    return names;
-}
-
-expression_ref add_prefix(const string& prefix, const expression_ref& E)
-{
-    std::set<string> names = find_named_parameters(E);
-
-    expression_ref E2 = E;
-    for(const auto& name: names)
-	E2 = substitute(E2, parameter(name), parameter(prefix+"."+name));
-
-    return E2;
-}
-
 /* Legal terms are:
 
    T,U,V -> x
@@ -167,7 +88,9 @@ expression_ref add_prefix(const string& prefix, const expression_ref& E)
 bool is_WHNF(const expression_ref& E)
 {
     int type = E.head().type();
-    if (E.size())
+    if (not E)
+        return false;
+    else if (E.size())
     {
 	if (type == lambda_type or type == lambda2_type or type == constructor_type) 
 	    return true;
@@ -183,26 +106,9 @@ bool is_WHNF(const expression_ref& E)
     }
 }
 
-bool is_parameter(const expression_ref& E)
-{
-    return E.is_a<parameter>();
-}
-
-bool is_modifiable(const expression_ref& E)
-{
-    bool result = E.head().type() == modifiable_type;
-    assert(result == E.head().is_a<modifiable>());
-    return result;
-}
-
-bool is_reg_var(const expression_ref& E)
-{
-    return E.is_a<reg_var>();
-}
-
 bool is_reglike(const expression_ref& E)
 {
-    return is_var(E) or is_parameter(E) or is_modifiable(E) or is_reg_var(E) or E.is_index_var();
+    return is_var(E) or is_modifiable(E) or is_reg_var(E) or E.is_index_var();
 }
 
 expression_ref launchbury_unnormalize(const expression_ref& E)
@@ -284,21 +190,20 @@ expression_ref launchbury_unnormalize(const expression_ref& E)
 
 	    for(int i=decls.size()-1; i>=0; i--)
 	    {
-		auto& x = decls[i].first;
-		std::set<var> free = get_free_indices(decls[i].second);
+		auto [x,E] = decls[i];
+		std::set<var> free = get_free_indices(E);
 
 		// if x references itself then don't substitute it.
 		if (free.count(x)) continue;
 	
 		changed = true;
 	
-		auto decl = decls[i];
 		decls.erase(decls.begin() + i);
 	
 		// substitute for the value of this variable in T and in the remaining bodies;
-		for(int j=0;j<decls.size();j++)
-		    decls[j].second = substitute(decls[j].second, decl.first, decl.second);
-		body = substitute(body, decl.first, decl.second);
+		for(auto& [x2,E2]: decls)
+		    E2 = substitute(E2, x, E);
+		body = substitute(body, x, E);
 	    }
 	}
 
@@ -327,27 +232,3 @@ expression_ref parse_object(const string& s)
     else
 	throw myexception()<<"Can't convert '"<<s<<"' to bool, int, double, or string!";
 }
-
-vector<double> vec_to_double(const std::vector<expression_ref>& v)
-{
-    vector<double> vv(v.size());
-    for(int i=0;i<v.size();i++)
-	vv[i] = v[i].as_double();
-    return vv;
-}
-
-vector<int> vec_to_int(const std::vector<expression_ref>& v)
-{
-    vector<int> vv(v.size());
-    for(int i=0;i<v.size();i++)
-	vv[i] = v[i].as_int();
-    return vv;
-}
-
-vector<char> vec_to_char(const std::vector<expression_ref>& v)
-{
-    vector<char> vv(v.size());
-    for(char i=0;i<v.size();i++)
-	vv[i] = v[i].as_char();
-    return vv;
-}
