@@ -1,6 +1,7 @@
 #include <iostream>
 #include "graph_register.H"
 #include "util/assert.hh"
+#include "util/range.H" // for remove_element( )
 
 using std::vector;
 using std::pair;
@@ -11,52 +12,68 @@ using std::endl;
 long total_destroy_token = 0;
 long total_release_knuckle = 0;
 
+token_type reverse(token_type type)
+{
+    switch(type)
+    {
+    case token_type::none:                std::abort();
+    case token_type::root:                std::abort();
+    case token_type::merged:              return token_type::merged;
+    case token_type::set:                 return token_type::reverse_set;
+    case token_type::set_unshare:         return token_type::reverse_set_unshare;
+    case token_type::execute:             return token_type::reverse_execute;
+    case token_type::reverse_set:         return token_type::set;
+    case token_type::reverse_set_unshare: return token_type::set_unshare;
+    case token_type::reverse_execute:     return token_type::execute;
+    }
+    std::abort();
+}
+
+std::ostream& operator<<(std::ostream& o, token_type type)
+{
+    switch(type)
+    {
+    case token_type::none:                o<<"none";break;
+    case token_type::root:                o<<"root";break;
+    case token_type::merged:              o<<"merged";break;
+    case token_type::set:                 o<<"set";break;
+    case token_type::set_unshare:         o<<"set_unshare";break;
+    case token_type::execute:             o<<"execute";break;
+    case token_type::reverse_set:         o<<"reverse_set";break;
+    case token_type::reverse_set_unshare: o<<"reverse_set_unshare";break;
+    case token_type::reverse_execute:     o<<"reverse_execute";break;
+    default:                              std::abort();
+    }
+    return o;
+}
+
+
 void reg_heap::destroy_all_computations_in_token(int t)
 {
     auto& delta_step = tokens[t].delta_step();
-    auto& delta_result = tokens[t].delta_result();
 
     // Remove use back-edges
-    for(auto p: delta_step)
+    for(auto& [_,s]: delta_step)
     {
-	int s = p.second;
-	if (s > 0)
-	{
-	    for(int r: steps[s].created_regs)
-	    {
-		// Truncating access(r) here deallocates RAM used by (for example) cached conditional likelihoods.
-		// Since we now ensure that any steps/result for these regs must be in this token, we can actually
-		//   deallocate the regs here instead of just waiting for GC to eliminate them.
-		regs.access(r).created_by = {0,{}};
-		reclaim_used(r);
-	    }
-	    steps[s].created_regs.clear();
-	    clear_back_edges_for_step(s);
-	}
+        if (s > 0)
+        {
+            for(int r: steps[s].created_regs)
+            {
+                // Unsharing created regs in unshare_regs( ) ensures that any steps for created regs must be in this token.
+                // This allows us to reclaim allocated regs here instead of just waiting for GC to eliminate them.
+
+                // Clearing the reg deallocates RAM used by (For example) cached conditional likelihoods.
+
+                clear_back_edges_for_reg(r,false);    // We don't need to adjust steps[s].created_regs, since we will destroy steps[s].
+                reclaim_used(r);                      // This clears the reg.
+            }
+            clear_back_edges_for_step(s);             // This clears steps[s].created_regs.
+            steps.reclaim_used(s);                    // This clears the step
+        }
     }
 
-    // Remove call back-edges
-    for(auto p: delta_result)
-    {
-	int rc = p.second;
-	if (rc > 0)
-	    clear_back_edges_for_result(rc);
-    }
-
-    for(auto p: delta_step)
-    {
-	int s = p.second;
-	if (s > 0)
-	    steps.reclaim_used(s);
-    }
     tokens[t].vm_step.clear();
 
-    for(auto p: delta_result)
-    {
-	int rc = p.second;
-	if (rc > 0)
-	    results.reclaim_used(rc);
-    }
     tokens[t].vm_result.clear();
 }
 
@@ -68,44 +85,52 @@ void reg_heap::release_tip_token(int t)
 
     total_destroy_token++;
 
-    // 1. Clear flags of results in the root token before destroying the root token!
-    if (is_root_token(t))
-	for(auto p: tokens[root_token].delta_result())
-	{
-	    int rc = p.second;
-	    if (rc > 0 and results[rc].flags.test(0))
-		dec_prior(rc);
-	}
-
-    // 2. Destroy computations in the token (this is an optimization)
-    destroy_all_computations_in_token(t);
-
-    // 3. Adjust the token tree
+    // 1. Adjust the token tree
     int parent = parent_token(t);
 
     unused_tokens.push_back(t);
   
     if (parent != -1)
     {
-	// mark token for this context unused
-	assert(not is_root_token(t));
-	assert(tokens.size() - unused_tokens.size() > 0);
+        // mark token for this context unused
+        assert(not is_root_token(t));
+        assert(tokens.size() - unused_tokens.size() > 0);
 
-	int index = remove_element(tokens[parent_token(t)].children, t);
-	assert(index != -1);
-	tokens[t].parent = -1;
+        int index = remove_element(tokens[parent_token(t)].children, t);
+        assert(index != -1);
     }
     else
     {
-	assert(is_root_token(t));
-	root_token = -1;
-	assert(tokens.size() - unused_tokens.size() == 0);
+        assert(is_root_token(t));
+        root_token = -1;
+        pending_effect_steps.clear();
+        assert(tokens.size() - unused_tokens.size() == 0);
     }
 
-    // 4. Set the token to unused
-    tokens[t].used = false;
+    // 2. Set the token to unused
 
-    // 5. Make sure the token is empty
+    tokens[t].parent = -1;
+    tokens[t].used = false;
+    tokens[t].type = token_type::none;
+
+    // 3. Destroy the computations
+    try
+    {
+        destroy_all_computations_in_token(t);
+    }
+    catch (std::exception& e)
+    {
+        std::cerr<<"Exception thrown while releasing tip token!\n";
+        std::cerr<<e.what()<<"\n";
+        std::abort();
+    }
+    catch (...)
+    {
+        std::cerr<<"Exception thrown while releasing tip token!\n";
+        std::abort();
+    }
+
+    // 4. Make sure the token is empty
     assert(tokens[t].vm_step.empty());
     assert(tokens[t].vm_result.empty());
 }
@@ -130,45 +155,45 @@ void reg_heap::capture_parent_token(int t2)
     tokens[t2].parent = parent;
 }
 
-void load_map(const mapping& vm, vector<std::bitset<8>>& prog_temp)
+template <typename T>
+void load_map(const mapping<T>& vm, vector<bitmask_8>& prog_temp)
 {
-    for(auto p: vm.delta())
+    for(auto [r,_]: vm.delta())
     {
-	int r = p.first;
-	assert(not prog_temp[r].test(0));
-	prog_temp[r].set(0);
+        assert(not prog_temp[r].test(0));
+        prog_temp[r].set(0);
     }
 }
 
-void unload_map(const mapping& vm, vector<std::bitset<8>>& prog_temp)
+template<typename T>
+void unload_map(const mapping<T>& vm, vector<bitmask_8>& prog_temp)
 {
-    for(auto p: vm.delta())
+    for(auto [r,_]: vm.delta())
     {
-	int r = p.first;
-	assert(prog_temp[r].test(0));
-	prog_temp[r].reset(0);
+        assert(prog_temp[r].test(0));
+        prog_temp[r].reset(0);
     }
 }
 
-void merge_split_mapping_(mapping& vm1, mapping& vm2, vector<std::bitset<8>>& prog_temp)
+template <typename T>
+void merge_split_mapping_(mapping<T>& vm1, mapping<T>& vm2, vector<bitmask_8>& prog_temp)
 {
     for(int i=0;i<vm1.delta().size();)
     {
-	int r = vm1.delta()[i].first;
-	int v = vm1.delta()[i].second;
+        auto [r,v] = vm1.delta()[i];
 
-	if (not prog_temp[r].test(0))
-	{
-	    vm2.add_value(r,v);
-	    prog_temp[r].set(0);
-	    vm1.erase_value_at(i);
-	}
-	else
-	    i++;
+        if (not prog_temp[r].test(0))
+        {
+            vm2.add_value(r,v);
+            prog_temp[r].set(0);
+            vm1.erase_value_at(i);
+        }
+        else
+            i++;
     }
 }
 
-// This function splits handles the composition of deltas: Delta1 o Delta2
+// This function handles the composition of deltas: Delta1 o Delta2
 //   It is only ever used to remove knuckle tokens (see tokens.cc).
 // Given mapping (m1,v1) followed by (m2,v2), compute a combined mapping for (m1,v1)+(m2,v2) -> (m2,v2)
 // and a mapping (m1,v1)-(m2,v2)->(m1,v1) for things that now are unused.
@@ -176,37 +201,35 @@ void reg_heap::merge_split_mappings(const vector<int>& knuckle_tokens)
 {
     if (knuckle_tokens.empty()) return;
 
+    // This is the child that will survive, versus the knuckles that will be deleted.
     int child_token = tokens[knuckle_tokens[0]].children[0];
 
+    tokens[child_token].type = token_type::merged;
+    for(int t: knuckle_tokens)
+    {
+        assert(tokens[t].type != token_type::set);
+        tokens[t].type = token_type::merged;
+    }
+
     load_map(tokens[child_token].vm_result, prog_temp);
     for(int t: knuckle_tokens)
     {
-	assert(token_is_used(t));
-	assert(not tokens[t].is_referenced());
-	assert(tokens[t].children.size() == 1);
+        assert(token_is_used(t));
+        assert(not tokens[t].is_referenced());
+        assert(tokens[t].children.size() == 1);
 
-	assert(tokens[t].version <= tokens[child_token].version);
-
-	// The child token (t2) needs to be up-to-date with respect to the parent token.
-	assert(tokens[t].version <= tokens[child_token].version);
-
-	merge_split_mapping_(tokens[t].vm_result, tokens[child_token].vm_result, prog_temp);
+        merge_split_mapping_(tokens[t].vm_result, tokens[child_token].vm_result, prog_temp);
     }
     unload_map(tokens[child_token].vm_result, prog_temp);
 
     load_map(tokens[child_token].vm_step, prog_temp);
     for(int t: knuckle_tokens)
     {
-	assert(token_is_used(t));
-	assert(not tokens[t].is_referenced());
-	assert(tokens[t].children.size() == 1);
+        assert(token_is_used(t));
+        assert(not tokens[t].is_referenced());
+        assert(tokens[t].children.size() == 1);
 
-	assert(tokens[t].version <= tokens[child_token].version);
-
-	// The child token (t2) needs to be up-to-date with respect to the parent token.
-	assert(tokens[t].version <= tokens[child_token].version);
-
-	merge_split_mapping_(tokens[t].vm_step, tokens[child_token].vm_step, prog_temp);
+        merge_split_mapping_(tokens[t].vm_step, tokens[child_token].vm_step, prog_temp);
     }
     unload_map(tokens[child_token].vm_step, prog_temp);
 }
@@ -220,22 +243,22 @@ int reg_heap::release_knuckle_tokens(int child_token)
 
     while (true)
     {
-	t = tokens[t].parent;
-	if (t != root_token and not tokens[t].is_referenced() and tokens[t].children.size() == 1)
-	    knuckle_path.push_back(t);
-	else
-	    break;
+        t = tokens[t].parent;
+        if (t != root_token and not tokens[t].is_referenced() and tokens[t].children.size() == 1)
+            knuckle_path.push_back(t);
+        else
+            break;
     }
 
     merge_split_mappings(knuckle_path);
 
     for(int t2: knuckle_path)
     {
-	capture_parent_token(child_token);
+        capture_parent_token(child_token);
 
-	total_release_knuckle++;
+        total_release_knuckle++;
 
-	release_tip_token(t2);
+        release_tip_token(t2);
     }
 
     return tokens[child_token].parent;
@@ -247,12 +270,12 @@ int reg_heap::release_unreferenced_tips(int t)
 
     while(t != -1 and not tokens[t].is_referenced() and tokens[t].children.empty())
     {
-	int parent = parent_token(t);
+        int parent = parent_token(t);
 
-	// clear only the mappings that were actually updated here.
-	release_tip_token(t);
+        // clear only the mappings that were actually updated here.
+        release_tip_token(t);
 
-	t = parent;
+        t = parent;
     }
 
     return t;
@@ -265,6 +288,11 @@ bool reg_heap::is_terminal_token(int t) const
     return tokens[t].children.empty();
 }
 
+int reg_heap::get_root_token() const
+{
+    return root_token;
+}
+
 bool reg_heap::is_root_token(int t) const
 {
     assert(token_is_used(t));
@@ -287,10 +315,10 @@ int reg_heap::degree_of_token(int t) const
 {
     int degree = children_of_token(t).size();
     if (not is_root_token(t))
-	degree++;
+        degree++;
     return degree;
 }
-  
+
 
 bool reg_heap::token_is_used(int t) const
 {
@@ -298,7 +326,17 @@ bool reg_heap::token_is_used(int t) const
     return tokens[t].used;
 }
 
-int reg_heap::make_child_token(int t)
+vector<int> reg_heap::get_used_tokens() const
+{
+    vector<int> toks;
+    for(int t=0;t<tokens.size();t++)
+        if (token_is_used(t))
+            toks.push_back(t);
+    return toks;
+}
+
+
+int reg_heap::make_child_token(int t, token_type type)
 {
 #ifdef DEBUG_MACHINE
     check_used_regs();
@@ -306,7 +344,7 @@ int reg_heap::make_child_token(int t)
 
     assert(tokens[t].used);
 
-    int t2 = get_unused_token();
+    int t2 = get_unused_token(type);
 
     // assert(temp.empty());
 
@@ -316,8 +354,6 @@ int reg_heap::make_child_token(int t)
 
     tokens[t].children.push_back(t2);
 
-    tokens[t2].version = tokens[t].version;
-
 #ifdef DEBUG_MACHINE
     check_used_regs();
 #endif
@@ -342,12 +378,12 @@ void reg_heap::switch_to_context(int c1, int c2)
     check_tokens();
 }
 
-int reg_heap::switch_to_child_token(int c)
+int reg_heap::switch_to_child_token(int c, token_type type)
 {
     check_tokens();
 
     int t1 = token_for_context(c);
-    int t2 = make_child_token(t1);
+    int t2 = make_child_token(t1, type);
     switch_to_token(c, t2);
 
     check_tokens();
@@ -400,13 +436,18 @@ int reg_heap::copy_context(int c)
     return c2;
 }
 
+long total_create_context1 = 0;
+long total_create_context2 = 0;
+
 int reg_heap::get_new_context()
 {
+    total_create_context2++;
+
     // Add an unused context if we are missing one
     if (unused_contexts.empty())
     {
-	unused_contexts.push_back(get_n_contexts());
-	token_for_context_.push_back(-1);
+        unused_contexts.push_back(get_n_contexts());
+        token_for_context_.push_back(-1);
     }
 
     // Get a new context index and check it has no token
@@ -417,14 +458,16 @@ int reg_heap::get_new_context()
     return c;
 }
 
-int reg_heap::get_unused_context()
+int reg_heap::get_first_context()
 {
     int c = get_new_context();
   
-    set_token_for_context(c, get_unused_token());
+    set_token_for_context(c, get_unused_token(token_type::root));
 
     check_tokens();
 
+    evaluate_program(c);
+
     return c;
 }
 
@@ -458,13 +501,13 @@ void reg_heap::release_context(int c)
  * 3. If the reg was 
  */
 
-int reg_heap::get_unused_token()
+int reg_heap::get_unused_token(token_type type)
 {
     if (unused_tokens.empty())
     {
-	unused_tokens.push_back(get_n_tokens());
-	tokens.push_back(Token());
-	total_tokens = tokens.size();
+        unused_tokens.push_back(get_n_tokens());
+        tokens.push_back(Token());
+        total_tokens = tokens.size();
     }
 
     int t = unused_tokens.back();
@@ -473,14 +516,15 @@ int reg_heap::get_unused_token()
     assert(not token_is_used(t));
 
     tokens[t].used = true;
+    tokens[t].type = type;
 
     if (root_token == -1)
     {
-	assert(tokens.size() - unused_tokens.size() == 1);
-	root_token = t;
+        assert(tokens.size() - unused_tokens.size() == 1);
+        root_token = t;
     }
     else
-	assert(tokens.size() - unused_tokens.size() > 1);
+        assert(tokens.size() - unused_tokens.size() > 1);
 
     assert(tokens[t].parent == -1);
     assert(tokens[t].children.empty());
