@@ -2,8 +2,9 @@
 #include "computation/machine/args.H"
 #include "computation/machine/graph_register.H"
 #include "math/exponential.H"
-#include "myexception.H"
-#include "expression/expression.H"
+#include "util/myexception.H"
+#include "expression/index_var.H"
+#include "expression/constructor.H"
 #include "expression/lambda.H"
 #include "expression/case.H"
 #include "expression/var.H"
@@ -103,7 +104,7 @@ closure apply_op(OperationArgs& Args)
     C.exp = peel_n_lambdas(C.exp, n_args_applied);
     for(int i=0;i<n_args_applied;i++)
     {
-	int arg = Args.current_closure().lookup_in_env( Args.reference(i+1).as_index_var());
+	int arg = Args.current_closure().reg_for_slot(i+1);
 	C.Env.push_back(arg);
     }
 
@@ -119,7 +120,7 @@ closure apply_op(OperationArgs& Args)
 	vector<expression_ref> args;
 	for(int i=n_args_needed;i<n_args_given;i++)
 	{
-	    int arg = Args.current_closure().lookup_in_env( Args.reference(i+1).as_index_var() );
+	    int arg = Args.current_closure().reg_for_slot(i+1);
 	    Env.push_back(arg);
 
 	    args.push_back(index_var(n_args_given - i -1));
@@ -195,12 +196,7 @@ closure case_op(OperationArgs& Args)
 		result.exp = this_body;
 	
 		for(int j=0;j<object.exp.size();j++)
-		{
-		    // Don't do a dynamic cast here!
-		    int index = object.exp.sub()[j].as_index_var();
-	  
-		    result.Env.push_back( object.lookup_in_env( index ) );
-		}
+		    result.Env.push_back( object.reg_for_slot(j) );
 	    }
 	}
     }
@@ -288,10 +284,33 @@ closure let_op(OperationArgs& Args)
     return C;
 }
 
-closure modifiable_op(OperationArgs&)
+closure join_op(OperationArgs& Args)
 {
-    // A modifiable has a result that is not computed by reducing an expression.
-    //       The result must be set.  Therefore, complain if the result is missing.
+    Args.evaluate_slot_to_reg(0);
+    int R = Args.evaluate_slot_to_reg(1);
 
-    throw myexception()<<"Evaluating modifiable with no result.";
+    return {index_var(0),{R}};
+}
+
+closure seq_op(OperationArgs& Args)
+{
+    Args.evaluate_slot_force(0);
+
+    int R = Args.current_closure().reg_for_slot(1);
+
+    return {index_var(0),{R}};
+}
+
+bool is_seq(const expression_ref& E)
+{
+    bool result = E.head().type() == seq_type;
+    assert(result == E.head().is_a<Seq>());
+    return result;
+}
+
+bool is_join(const expression_ref& E)
+{
+    bool result = E.head().type() == join_type;
+    assert(result == E.head().is_a<Join>());
+    return result;
 }
