@@ -24,15 +24,86 @@
 ///
 
 #include "alignment-sums.H"
-#include "prior.H"
-#include "substitution/substitution.H"
-#include "util.H"
+#include <math.h>                                   // for pow
+#include <stdlib.h>                                 // for abort
+#include <algorithm>                                // for find
+#include <boost/dynamic_bitset/dynamic_bitset.hpp>  // for dynamic_bitset
+#include <iostream>                                 // for operator<<, ostream
+#include <string>                                   // for operator<<
+#include "alignment/alignment.H"                    // for operator<<
+#include "dp/2way.H"                                // for pairwise_alignment_t
+#include "dp/A2_states.H"                           // for E, S
+#include "dp/dp-engine.H"                           // for DPengine
+#include "imodel/imodel.H"                          // for PairHMM
+#include "models/TreeInterface.H"                   // for TreeInterface
+#include "models/parameters.H"                      // for data_partition
+#include "substitution/substitution.H"              // for other_subst
+#include "util/assert.hh"                           // for assert
+#include "util/matrix.H"                            // for matrix
+#include "util/myexception.H"                       // for show_stack_trace
+#include "util/set.H"                               // for includes
+
+extern int log_verbose;
 
 using boost::dynamic_bitset;
 using std::vector;
 using std::cerr;
 using std::endl;
 
+matrix<int> get_path_counts(const pairwise_alignment_t& a)
+{
+  using namespace A2;
+
+  matrix<int> counts(5,5,0);
+
+  int prev = A2::states::S;
+  for(int i=0;i<a.size();i++)
+  {
+      counts(prev, a.get_state(i))++;
+      prev = a.get_state(i);
+  }
+  counts(prev, A2::states::E)++;
+
+  return counts;
+}
+
+/// Probability of a pairwise alignment
+log_double_t prior_branch_from_counts(const matrix<int>& counts,const indel::PairHMM& Q)
+{
+  using namespace A2;
+
+  log_double_t P=1;
+
+  // Account for S-? start probability
+  for(int i=0;i<Q.size2();i++)
+    if (counts(states::S,i))
+      P *= Q.start(i);
+
+  // Account for the mass of transitions
+  for(int i=0;i<3;i++)
+    for(int j=0;j<3;j++) {
+      log_double_t Qij = Q(i,j);
+      // FIXME - if we propose really bad indel parameters, we can get log(Q_ij) where Qij == 0
+      if (counts(i,j))
+	P *= pow(Qij,counts(i,j));
+    }
+  
+  // Account for ?-E end probability
+  if (not counts(states::S,states::E))
+    for(int i=0;i<Q.size1();i++)
+      if (counts(i,states::E))
+       P *= Q(i,states::E);
+
+  return P;
+}
+
+/// Probability of a pairwise alignment
+log_double_t prior_branch(const pairwise_alignment_t& a,const indel::PairHMM& Q) 
+{
+  matrix<int> counts = get_path_counts(a);
+
+  return prior_branch_from_counts(counts,Q);
+}
 log_double_t other_subst(const data_partition& P, const vector<int>& nodes) 
 {
     return substitution::other_subst(P,nodes);
