@@ -28,24 +28,29 @@
 /// \author Benjamin Redelings
 /// 
 
-#include <iostream>
-#include <algorithm>
-
 #include "mcmc.H"
-#include "sample.H"
-#include "myexception.H"
-#include "rng.H"
-#include "util.H"
-#include "util-random.H"
-
-#include "../setup.H"           // for letter_counts( )
-
-#include "monitor.H"
-#include "proposals.H"
-#include "alignment/alignment-util.H"
-#include "computation/expression/expression.H"
-
-#include "slice-sampling.H"
+#include "util/assert.hh"                           // for assert
+#include <math.h>                                   // for log
+#include <stdlib.h>                                 // for abs, abort
+#include <algorithm>                                // for copy, max, fill
+#include <boost/smart_ptr/intrusive_ptr.hpp>        // for intrusive_ptr
+#include <iostream>                                 // for operator<<, ostream
+#include <iterator>                                 // for valarray
+#include <memory>                                   // for allocator_traits<...
+#include <utility>                                  // for pair
+#include "computation/expression/expression_ref.H"  // for expression_ref
+#include "computation/object.H"                     // for intrusive_ptr_rel...
+#include "models/TreeInterface.H"                   // for TreeInterface
+#include "models/model.H"                           // for Model, show_param...
+#include "models/parameters.H"                      // for Parameters, accep...
+#include "proposals.H"                              // for Proposal2, Proposal
+#include "slice-sampling.H"                         // for integer_random_va...
+#include "util/math/log-double.H"                   // for operator<<, log_d...
+#include "util/myexception.H"                       // for myexception
+#include "util/permute.H"                           // for randomize
+#include "util/range.H"                             // for sum
+#include "util/rng.H"                               // for poisson, uniform
+#include "util/string/split.H"                      // for split
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -57,6 +62,8 @@
 namespace mpi = boost::mpi;
 #endif
 
+extern int log_verbose;
+
 using std::endl;
 
 namespace MCMC {
@@ -333,11 +340,12 @@ namespace MCMC {
 
 	iterations++;
 
+        P->evaluate_program();
 	owned_ptr<Model> P2 = P;
 
 	log_double_t ratio = 1;
 	try {
-	    ratio = (*proposal)(*P2);
+	    ratio = proposal(*P2);
 	}
 	catch (myexception& e)
 	{
@@ -346,62 +354,12 @@ namespace MCMC {
 	    e.prepend(o.str());
 	    throw;
 	}
-    
 
-	int n = 1;
-	Proposal2* p2 = dynamic_cast<Proposal2*>(&(*proposal));
-	int n_indices = -1;
-	if (p2) {
-	    n_indices = p2->get_indices().size();
-	    n = 2;
-	}
-	Result result(n);
-
-#ifndef NDEBUG
-	// Check that we have not strayed outside the bounds.
-	for(int i=0;i<P->n_parameters();i++)
-	{
-	    if (not P->parameter_is_modifiable_reg(i)) continue;
-
-	    if (not P->get_parameter_value(i).is_double()) continue;
-
-	    if (not P->has_bounds(i)) continue;
-	
-	    Bounds<double> range = P->get_bounds(i);
-	    if (not range.in_range(P->get_parameter_value(i).as_double()))
-		throw myexception()<<"Parameter "<<P->parameter_name(i)<<" = "<<P->get_parameter_value(i).as_double()<<" is NOT in range "<<range;
-	    if (not range.in_range(P2->get_parameter_value(i).as_double()))
-		throw myexception()<<"Parameter "<<P->parameter_name(i)<<" = "<<P->get_parameter_value(i).as_double()<<" is NOT in range "<<range;
-	}
-#endif
+	Result result(1);
 
 	// Accept or Reject
-	if (accept_MH(*P,*P2,ratio)) {
+	if (perform_MH(*P, *P2, ratio))
 	    result.totals[0] = 1;
-	    if (n == 2) {
-		int first_index = p2->get_indices()[0];
-		if (n_indices == 1 and P->get_parameter_value(first_index).is_double()) 
-		{
-		    double v1 = P->get_parameter_value(first_index).as_double();
-		    double v2 = P2->get_parameter_value(first_index).as_double();
-		    //      cerr<<"v1 = "<<v1<<"   v2 = "<<v2<<"\n";
-		    result.totals[1] = std::abs(v2-v1);
-		}
-		else if (n_indices > 1 and P->get_parameter_value(first_index).is_double()) //currently this can only be a dirichlet proposal
-		{
-		    double total = 0;
-		    for(int i=0;i<n_indices;i++) 
-		    {
-			int j = p2->get_indices()[i];
-			double v1 = P->get_parameter_value(j).as_double();
-			double v2 = P2->get_parameter_value(j).as_double();
-			total += std::abs(log(v1/v2));
-		    }
-		    result.totals[1] = total;
-		}
-	    }
-	    P = P2;
-	}
 
 	Stats.inc(name,result);
     }
@@ -413,7 +371,7 @@ namespace MCMC {
 	iterations++;
 
 	//------------- Find new value --------------//
-	if (log_verbose >= 3)
+	if (log_verbose >= 4)
 	{
 	    show_parameters(std::cerr,P);
 	    std::cerr<<P.probability()<<" = "<<P.likelihood()<<" + "<<P.prior();
@@ -432,7 +390,7 @@ namespace MCMC {
 		W = 0.95*W + 0.05*W_predicted;
 	}
 
-	if (log_verbose >= 3)
+	if (log_verbose >= 4)
 	{
 	    show_parameters(std::cerr,P);
 	    std::cerr<<P.probability()<<" = "<<P.likelihood()<<" + "<<P.prior()<<std::endl;
@@ -488,13 +446,13 @@ namespace MCMC {
 	 total_movement(0)
     {}
 
-    void Modifiable_Slice_Move::iterate(owned_ptr<Model>& P,MoveStats& Stats,int)
+    void Random_Variable_Slice_Move::iterate(owned_ptr<Model>& P,MoveStats& Stats,int)
     {
 	if (log_verbose >= 3) clog<<" [modifiable slice] move = "<<m_index<<endl;
 
 	double v1 = P->get_modifiable_value(m_index).as_double();
 
-	modifiable_slice_function logp(*P, bounds, m_index);
+	random_variable_slice_function logp(*P, bounds_, m_index);
 
 	double v2 = sample(*P,logp,v1);
 
@@ -507,24 +465,24 @@ namespace MCMC {
 	Stats.inc(name,result);
     }
 
-    Modifiable_Slice_Move::Modifiable_Slice_Move(const string& s,int m,
-						 const Bounds<double>& b, double W_)
-	:Slice_Move(s,W_), m_index(m), bounds(b)
+    Random_Variable_Slice_Move::Random_Variable_Slice_Move(const string& s,int m,
+						 const bounds<double>& b, double W_)
+	:Slice_Move(s,W_), m_index(m), bounds_(b)
     {}
 
-    Modifiable_Slice_Move::Modifiable_Slice_Move(const string& s, const string& v,int m,
-						 const Bounds<double>& b, double W_)
-	:Slice_Move(s,v,W_), m_index(m), bounds(b)
+    Random_Variable_Slice_Move::Random_Variable_Slice_Move(const string& s, const string& v,int m,
+						 const bounds<double>& b, double W_)
+	:Slice_Move(s,v,W_), m_index(m), bounds_(b)
     {}
 
-    void Integer_Modifiable_Slice_Move::iterate(owned_ptr<Model>& P,MoveStats& Stats,int)
+    void Integer_Random_Variable_Slice_Move::iterate(owned_ptr<Model>& P,MoveStats& Stats,int)
     {
 	if (log_verbose >= 3) clog<<" [integer modifiable slice] move = "<<m_index<<endl;
 
 	int v1 = P->get_modifiable_value(m_index).as_int();
 	double x1 = double(v1)+uniform();
 
-	integer_modifiable_slice_function logp(*P, bounds, m_index);
+	integer_random_variable_slice_function logp(*P, bounds_, m_index);
 
 	double x2 = sample(*P,logp,x1);
 
@@ -538,40 +496,16 @@ namespace MCMC {
 	Stats.inc(name,result);
     }
 
-    Integer_Modifiable_Slice_Move::Integer_Modifiable_Slice_Move(const string& s,int m,
-								 const Bounds<int>& b, double W_)
-	:Slice_Move(s,W_), m_index(m), bounds(b)
+    Integer_Random_Variable_Slice_Move::Integer_Random_Variable_Slice_Move(const string& s,int m,
+								 const bounds<int>& b, double W_)
+	:Slice_Move(s,W_), m_index(m), bounds_(b)
     {}
 
-    Integer_Modifiable_Slice_Move::Integer_Modifiable_Slice_Move(const string& s, const string& v,int m,
-								 const Bounds<int>& b, double W_)
-	:Slice_Move(s,v,W_), m_index(m), bounds(b)
+    Integer_Random_Variable_Slice_Move::Integer_Random_Variable_Slice_Move(const string& s, const string& v,int m,
+								 const bounds<int>& b, double W_)
+	:Slice_Move(s,v,W_), m_index(m), bounds_(b)
     {}
 
-    void Dirichlet_Slice_Move::iterate(owned_ptr<Model>& P,MoveStats& Stats,int)
-    {
-	if (log_verbose >= 3) clog<<" [dirichlet slice] move"<<endl;
-
-	double v1 = P->get_parameter_value(indices[n]).as_double();
-	constant_sum_slice_function slice_levels_function(*P,indices,n);
-
-	double v2 = sample(*P,slice_levels_function,v1);
-
-	//---------- Record Statistics - -------------//
-	Result result(2);
-	vector<double> x = vec_to_double(P->get_parameter_values(indices));
-	double total = sum(x);
-	double factor = (total - v2)/(total-v1);
-	result.totals[0] = std::abs(log(v2/v1)) + (indices.size()-1)*(std::abs(log(factor)));
-	result.totals[1] = slice_levels_function.count;
-
-	Stats.inc(name,result);
-    }
-
-    Dirichlet_Slice_Move::Dirichlet_Slice_Move(const string& s, const vector<int>& indices_, int n_)
-	:Slice_Move(s,0.2/indices_.size()),indices(indices_),n(n_)
-    { }
-
     void Dirichlet_Modifiable_Slice_Move::iterate(owned_ptr<Model>& P,MoveStats& Stats,int)
     {
 	if (log_verbose >= 3) clog<<" [dirichlet modifiable slice] move"<<endl;
@@ -583,7 +517,7 @@ namespace MCMC {
 
 	//---------- Record Statistics - -------------//
 	Result result(2);
-	vector<double> x = vec_to_double(P->get_modifiable_values(indices));
+	auto x = (vector<double>) P->get_modifiable_values(indices);
 	double total = sum(x);
 	double factor = (total - v2)/(total-v1);
 	result.totals[0] = std::abs(log(v2/v1)) + (indices.size()-1)*(std::abs(log(factor)));
@@ -662,22 +596,6 @@ namespace MCMC {
     }
 
 
-    void IOMove::iterate(owned_ptr<Model>& P, MoveStats& M)
-    {
-	iterate(P,M,0);
-    }
-
-    void IOMove::iterate(owned_ptr<Model>& P, MoveStats& /* M */, int)
-    {
-	if (log_verbose >= 3) clog<<" [IO Move] move = "<<head<<endl;
-
-	P->perform_transition_kernel(head);
-    }
-
-    IOMove::IOMove(int h)
-	:Move("IOMove"+convertToString(h)),head(h) 
-    { }
-  
     void MoveEach::add(double l,const MoveArg& m,bool enabled) {
 	MoveGroupBase::add(l,m,enabled);
 
@@ -1066,7 +984,7 @@ namespace MCMC {
 	    if (subsample <= 0) subsample = 2*int(log(t.n_leaves()))+1;
 
 	    if (alignment_burnin_iterations > 0)
-		PP->set_parameter_value(PP->find_parameter("*IModels.training"), bool_true);
+		PP->set_imodel_training(true);
 	}
 
 	//---------------- Run the MCMC chain -------------------//
@@ -1076,7 +994,7 @@ namespace MCMC {
 	    {
 		// Free temporarily fixed parameters at iteration 5
 		if (iterations == alignment_burnin_iterations)
-		    PP->set_parameter_value(PP->find_parameter("*IModels.training"), bool_false);
+		    PP->set_imodel_training(false);
 
 		// Change the temperature according to the pattern suggested
 		if (iterations < PP->PC->beta_series.size())
@@ -1102,6 +1020,7 @@ namespace MCMC {
 	    //------------------- move to new position -----------------//
 	    iterate(P,*this);
 
+            P->run_transition_kernels();
 
 #ifdef HAVE_MPI
 	    //------------------ Exchange Temperatures -----------------//
@@ -1114,6 +1033,8 @@ namespace MCMC {
 	    if (P.as<Parameters>())
 		exchange_adjacent_pairs(iterations,*P.as<Parameters>(),*this);
 #endif
+            if (log_verbose and iterations%100 == 0)
+                s_out<<(const MoveStats&)(*this);
 	}
 
 	s_out<<(const MoveStats&)(*this);
