@@ -86,7 +86,7 @@ shared_ptr<DParrayConstrained> sample_node_base(mutable_data_partition P,const v
     // Construct the states that are allowed for each emission pattern.
     for(int S2:Matrices->dp_order()) 
     {
-	unsigned int mask = (m123.state_emit[S2] & ~m123.hidden_bits).to_ulong();
+	auto mask = (m123.state_emit[S2] & ~m123.hidden_bits).raw();
     
 	// Hidden states never contradict an emission pattern.
 	if (not mask)
@@ -102,7 +102,7 @@ shared_ptr<DParrayConstrained> sample_node_base(mutable_data_partition P,const v
     // Determine which states are allowed to match other columns (c2)
     for(int c2=0;c2<a123_emit.size();c2++) 
     {
-	unsigned int mask=(a123_emit[c2]&~m123.hidden_bits).to_ulong();
+	auto mask=(a123_emit[c2]&~m123.hidden_bits).raw();
 	assert(mask);
     
 	Matrices->states(c2+1) = allowed_states_for_mask[mask];
@@ -117,19 +117,30 @@ shared_ptr<DParrayConstrained> sample_node_base(mutable_data_partition P,const v
     if (Matrices->Pr_sum_all_paths() <= 0.0)
     {
 	std::cerr<<"sample_node_base( ): All paths have probability 0!"<<std::endl;
-	return Matrices;
+
+        // Computing likelihoods when pairwise alignments disagree about internal sequence
+        // lengths can yield memory access errors.
+        for(int i=0;i<3;i++)
+        {
+            int b = P.t().find_branch(nodes[0],nodes[i+1]);
+            P.unset_pairwise_alignment(b);
+        }
     }
+    else
+    {
 
-    vector<int> path_g = Matrices->sample_path();
-    vector<int> path = Matrices->ungeneralize(path_g);
+        vector<int> path_g = Matrices->sample_path();
+        vector<int> path = Matrices->ungeneralize(path_g);
 
-    for(int i=0;i<3;i++) {
-	int b = P.t().find_branch(nodes[0],nodes[i+1]);
-	P.set_pairwise_alignment(b, get_pairwise_alignment_from_path(path, *Matrices, 3, i));
+        for(int i=0;i<3;i++)
+        {
+            int b = P.t().find_branch(nodes[0],nodes[i+1]);
+            P.set_pairwise_alignment(b, get_pairwise_alignment_from_path(path, *Matrices, 3, i));
+        }
+
+        assert(valid(P.A()));
     }
 
-    assert(valid(P.A()));
-
     return Matrices;
 }
 
@@ -163,7 +174,15 @@ int sample_node_multi(vector<Parameters>& p,const vector< vector<int> >& nodes_,
     for(int i=0;i<p.size();i++) {
 	for(int j=0;j<p[i].n_data_partitions();j++) 
 	    if (p[i][j].variable_alignment())
+            {
 		Matrices[i].push_back( sample_node_base(p[i][j],nodes[i]) );
+		if (Matrices[i].back()->Pr_sum_all_paths() <= 0.0)
+                {
+		    std::cerr<<"Pr = 0   i = "<<i<<"   j="<<j<<" \n";
+                    // Avoid calculating likelihoods and priors if the alignment is unset.
+                    return -1;
+                }
+            }
 	    else
 		Matrices[i].push_back( shared_ptr<DParrayConstrained>() );
     }
