@@ -17,21 +17,33 @@
   along with BAli-Phy; see the file COPYING.  If not see
   <http://www.gnu.org/licenses/>.  */
 
-#include <iostream>
-#include <cmath>
-
-#include "probability/choose.H"
-#include "util.H"
-#include "rng.H"
-#include "dp/3way.H"
-#include "dp/alignment-sums.H"
-#include "alignment/alignment-util.H"
-#include "alignment/alignment-util2.H"
-#include "alignment/alignment-constraint.H"
-#include <boost/shared_ptr.hpp>
-#include "dp/dp-matrix.H"
-#include "substitution/substitution.H"
-#include "util/assert.hh"
+#include <algorithm>                                // for sort, max
+#include "util/bitmask.H"                           // for bitmask
+#include <boost/dynamic_bitset/dynamic_bitset.hpp>  // for dynamic_bitset
+#include <iostream>                                 // for operator<<, endl
+#include <memory>                                   // for shared_ptr, __sha...
+#include <new>                                      // for bad_alloc
+#include <optional>                                 // for optional
+#include <string>                                   // for operator+, string
+#include <utility>                                  // for pair, move
+#include <vector>                                   // for vector, vector::s...
+#include "alignment/alignment-constraint.H"         // for boundaries_inters...
+#include "computation/object.H"                     // for intrusive_ptr_rel...
+#include "dp/2way.H"                                // for convert_to_bits
+#include "dp/3way.H"                                // for get_nodes_branch_...
+#include "dp/alignment-sums.H"                      // for sample_A3_multi_c...
+#include "dp/dp-engine.H"                           // for DPengine
+#include "dp/dp-matrix.H"                           // for DPmatrixConstrained
+#include "dp/hmm.H"                                 // for HMM::bitmask_t, HMM
+#include "models/TreeInterface.H"                   // for TreeInterface
+#include "models/model.H"                           // for show_parameters
+#include "models/parameters.H"                      // for Parameters, mutab...
+#include "probability/choose.H"                     // for choose_MH_P, choo...
+#include "substitution/substitution.H"              // for get_column_likeli...
+#include "util/assert.hh"                           // for assert
+#include "util/math/log-double.H"                   // for log_double_t, ope...
+#include "util/myexception.H"                       // for myexception
+#include "util/rng.H"                               // for uniform
 
 //Assumptions:
 //  a) we assume that the internal node is the parent sequence in each of the sub-alignments
@@ -40,10 +52,11 @@ using std::abs;
 using std::vector;
 using std::pair;
 using std::endl;
+using std::optional;
+using std::shared_ptr;
 using boost::dynamic_bitset;
-using boost::optional;
 
-boost::shared_ptr<DPmatrixConstrained>
+shared_ptr<DPmatrixConstrained>
 tri_sample_alignment_base(mutable_data_partition P, const vector<int>& nodes, const vector<HMM::bitmask_t>& a23,
 			  int /* bandwidth */)
 {
@@ -87,7 +100,7 @@ tri_sample_alignment_base(mutable_data_partition P, const vector<int>& nodes, co
     for(int i=0;i<3;i++)
 	branches[i] = t.find_branch(nodes[0],nodes[i+1]);
 
-    boost::shared_ptr<DPmatrixConstrained> 
+    shared_ptr<DPmatrixConstrained> 
 	Matrices(new DPmatrixConstrained(m123, std::move(dists1), std::move(dists23), P.WeightedFrequencyMatrix()));
     Matrices->emit1 = 1;
     Matrices->emit2 = 2|4;
@@ -100,7 +113,7 @@ tri_sample_alignment_base(mutable_data_partition P, const vector<int>& nodes, co
     // Construct the states that are allowed for each emission pattern.
     for(int S2: Matrices->dp_order())
     {
-	unsigned int mask = (m123.state_emit[S2] & Matrices->emit2).to_ulong();
+	auto mask = (m123.state_emit[S2] & Matrices->emit2).raw();
 
 	// Hidden states never contradict an emission pattern.
 	if (not mask) // m123.silent(S2))
@@ -118,7 +131,7 @@ tri_sample_alignment_base(mutable_data_partition P, const vector<int>& nodes, co
     // Determine which states are allowed to match (,c2)
     for(int c2=1;c2<Matrices->dists2.n_columns()-1;c2++) 
     {
-	unsigned int mask = (a23[c2-1]&Matrices->emit2).to_ulong();
+	auto mask = (a23[c2-1]&Matrices->emit2).raw();
 	mask >>= 1;
 	assert(mask);
 
@@ -151,19 +164,25 @@ tri_sample_alignment_base(mutable_data_partition P, const vector<int>& nodes, co
     // If the DP matrix ended up having probability 0, don't try to sample a path through it!
     if (Matrices->Pr_sum_all_paths() <= 0.0) 
     {
-#ifndef NDEBUG_DP
-	Matrices->clear();
-#endif
-	return Matrices;
+        // Computing likelihoods when pairwise alignments disagree about internal sequence
+        // lengths can yield memory access errors.
+        for(int i=0;i<3;i++)
+        {
+            int b = t.find_branch(nodes[0],nodes[i+1]);
+            P.unset_pairwise_alignment(b);
+        }
     }
+    else
+    {
+        vector<int> path_g = Matrices->sample_path();
 
-    vector<int> path_g = Matrices->sample_path();
+        vector<int> path = Matrices->ungeneralize(path_g);
 
-    vector<int> path = Matrices->ungeneralize(path_g);
-
-    for(int i=0;i<3;i++) {
-	int b = t.find_branch(nodes[0],nodes[i+1]);
-	P.set_pairwise_alignment(b, get_pairwise_alignment_from_path(path, *Matrices, 3, i));
+        for(int i=0;i<3;i++)
+        {
+            int b = t.find_branch(nodes[0],nodes[i+1]);
+            P.set_pairwise_alignment(b, get_pairwise_alignment_from_path(path, *Matrices, 3, i));
+        }
     }
 
 #ifdef NDEBUG_DP
@@ -201,7 +220,7 @@ vector<optional<vector<HMM::bitmask_t>>> A23_constraints(const Parameters& P, co
     return a23;
 }
 
-boost::shared_ptr<DPmatrixConstrained>
+shared_ptr<DPmatrixConstrained>
 tri_sample_alignment_base(mutable_data_partition P, const data_partition& P0,
 			  const vector<int>& nodes, const vector<int>& nodes0,
 			  int bandwidth)
@@ -268,6 +287,11 @@ log_double_t pr_sum_out_A_tri(Parameters P, const vector<optional<vector<HMM::bi
     {
 	if (P[j].variable_alignment()) {
 	    auto Matrices = tri_sample_alignment_base(P[j], nodes, *a23[j], -1);
+            if (Matrices->Pr_sum_all_paths() <= 0.0)
+            {
+                std::cerr<<"pr_sum_out_A_tri( ): All paths have probability 0!"<<std::endl;
+                return log_double_t(0.0);
+            }
 	    Pr *= Matrices->Pr_sum_all_paths();
 	    Pr *= pow(other_subst(P[j], nodes), P[j].get_beta());
 	    Pr *= other_prior(P[j], nodes);
@@ -279,7 +303,7 @@ log_double_t pr_sum_out_A_tri(Parameters P, const vector<optional<vector<HMM::bi
     return Pr;
 }
 
-void sample_A3_multi_calculation::run_dp()
+bool sample_A3_multi_calculation::run_dp()
 {
     assert(p.size() == nodes.size());
 
@@ -302,10 +326,13 @@ void sample_A3_multi_calculation::run_dp()
     for(int i=0;i<p.size();i++) 
     {
 	Matrices[i].resize(p[i].n_data_partitions());
-	for(int j=0;j<p[i].n_data_partitions();j++) {
+	for(int j=0;j<p[i].n_data_partitions();j++)
 	    if (p[i][j].variable_alignment())
+            {
 		Matrices[i][j] = compute_matrix(i,j);
-	}
+                if (Matrices[i][j]->Pr_sum_all_paths() <= 0.0)
+                    return false;
+            }
     }
 
     //-------- Calculate corrections to path probabilities ---------//
@@ -345,6 +372,7 @@ void sample_A3_multi_calculation::run_dp()
 		Pr[i] *= p[i][j].heated_likelihood();
     }
     assert(Pr[0] > 0.0);
+    return true;
 }
 
 void sample_A3_multi_calculation::set_proposal_probabilities(const vector<log_double_t>& r)
@@ -479,7 +507,7 @@ int sample_A3_multi_calculation::choose(bool correct)
     return C;
 }
 
-boost::shared_ptr<DPengine> sample_tri_multi_calculation::compute_matrix(int i, int j)
+shared_ptr<DPengine> sample_tri_multi_calculation::compute_matrix(int i, int j)
 {
     return tri_sample_alignment_base(p[i][j], p[0][j], nodes[i], nodes[0], bandwidth);
 }
@@ -496,15 +524,15 @@ int sample_tri_multi(vector<Parameters>& p,const vector< vector<int> >& nodes,
 		     const vector<log_double_t>& rho, bool do_OS,bool do_OP) 
 {
     try {
-	boost::shared_ptr<sample_A3_multi_calculation> tri;
+	shared_ptr<sample_A3_multi_calculation> tri;
 	if (uniform() < p[0].load_value("cube_fraction",0.0))
-	    tri = boost::shared_ptr<sample_A3_multi_calculation>(new sample_cube_multi_calculation(p, nodes, do_OS, do_OP));
+	    tri = shared_ptr<sample_A3_multi_calculation>(new sample_cube_multi_calculation(p, nodes, do_OS, do_OP));
 	else
-	    tri = boost::shared_ptr<sample_A3_multi_calculation>(new sample_tri_multi_calculation(p, nodes, do_OS, do_OP));
-	tri->run_dp();
+	    tri = shared_ptr<sample_A3_multi_calculation>(new sample_tri_multi_calculation(p, nodes, do_OS, do_OP));
+	bool ok = tri->run_dp();
 
 	// The DP matrix construction didn't work.
-	if (tri->Pr[0] <= 0.0) return -1;
+	if (not ok) return -1;
 
 	tri->set_proposal_probabilities(rho);
 
@@ -525,10 +553,10 @@ int sample_tri_multi(vector<Parameters>& p,const vector< vector<int> >& nodes,
 
 	//----------------- Part 1: Forward -----------------//
 	sample_tri_multi_calculation tri1(p, nodes, do_OS, do_OP, bandwidth);
-	tri1.run_dp();
+	bool ok = tri1.run_dp();
 
 	// The DP matrix construction didn't work.
-	if (tri1.Pr[0] <= 0.0) return -1;
+	if (not ok) return -1;
 
 	tri1.set_proposal_probabilities(rho);
 
@@ -550,10 +578,10 @@ int sample_tri_multi(vector<Parameters>& p,const vector< vector<int> >& nodes,
 	std::abort();
 
 	sample_tri_multi_calculation tri2(p2, nodes, do_OS, do_OP, bandwidth);
-	tri2.run_dp();
+	bool ok2 = tri2.run_dp();
 
 	// The DP matrix construction didn't work.
-	if (tri2.Pr[0] <= 0.0) return -1;
+	if (not ok2) return -1;
 
 	tri2.set_proposal_probabilities(rho);
 
@@ -626,21 +654,18 @@ bool tri_sample_alignment_branch(Parameters& P,
     return (C > 0);
 }
 
-bool tri_sample_alignment_and_parameter(Parameters& P,
-					int node1,int node2,int p_index,
-					double rho_,double v2)
+#include "mcmc/proposals.H"
+
+bool tri_sample_alignment_and_parameter(Parameters& P, int node1,int node2, const Proposal& propose)
 {
     //----------- Generate the Different Matrices ---------//
     vector<Parameters> p(2,P);
-    p[1].set_parameter_value(p_index,v2);
 
     vector< vector<int> > nodes (2, A3::get_nodes_branch_random(P.t(),node1,node2) );
 
-    vector<log_double_t> rho(2);
-    rho[0] = 1;
-    rho[1] = rho_;
+    auto rho = propose(p[1]);
 
-    int C = sample_tri_multi(p,nodes,rho,false,false);
+    int C = sample_tri_multi(p, nodes, {1.0, rho}, false, false);
 
     if (C != -1) {
 	P = p[C];
