@@ -28,8 +28,9 @@
 #include "util/assert.hh"
 #include "sample.H"
 #include "probability/choose.H"
-#include "util.H"
-#include "rng.H"
+#include "util/mapping.H"
+#include "util/rng.H"
+#include "dp/2way.H"
 #include "dp/5way.H"
 #include "dp/alignment-sums.H"
 #include "alignment/alignment-util.H"
@@ -38,14 +39,15 @@
 #include "dp/dp-array.H"
 
 using std::vector;
+using std::optional;
 using std::abs;
 using std::endl;
+using std::shared_ptr;
 
 using boost::dynamic_bitset;
-using boost::shared_ptr;
 
 shared_ptr<DParrayConstrained>
-sample_two_nodes_base(mutable_data_partition P, const data_partition& P0, const A5::hmm_order& order, const A5::hmm_order& order0)
+sample_two_nodes_base(mutable_data_partition P, const vector<HMM::bitmask_t>& a123456, const A5::hmm_order& order, const A5::hmm_order& order0)
 {
     assert(P.variable_alignment());
     HMM m12345 = A5::get_HMM(P,order);
@@ -56,9 +58,8 @@ sample_two_nodes_base(mutable_data_partition P, const data_partition& P0, const
 	I guess it means that we used (A0,T0,nodes0) to get an order.  We then keep that order
 	for use with (A[i],T[i],nodes[i]).  */
 
-    vector<HMM::bitmask_t> a123456 = A5::get_bitpath(P0, order0);
-    a123456 = remap_bitpath(a123456, compute_mapping(order0.nodes, order.nodes));
-    vector<HMM::bitmask_t> a1234 = remove_silent(a123456, m12345.all_bits() & ~m12345.hidden_bits);
+    auto a123456_remapped = remap_bitpath(a123456, compute_mapping(order0.nodes, order.nodes));
+    vector<HMM::bitmask_t> a1234 = remove_silent(a123456_remapped, m12345.all_bits() & ~m12345.hidden_bits);
 
     /*-------------- Create DP matrices ---------------*/
 
@@ -71,7 +72,7 @@ sample_two_nodes_base(mutable_data_partition P, const data_partition& P0, const
 
     for(int S2: Matrices->dp_order())
     {
-	unsigned int mask = (m12345.state_emit[S2] & ~m12345.hidden_bits).to_ulong();
+	auto mask = (m12345.state_emit[S2] & ~m12345.hidden_bits).raw();
 
 	// Hidden states never contradict an emission pattern.
 	if (not mask)
@@ -86,7 +87,7 @@ sample_two_nodes_base(mutable_data_partition P, const data_partition& P0, const
     // Determine which states are allowed to match (c2)
     for(int c2=0;c2<Matrices->size()-1;c2++) 
     {
-	unsigned mask=a1234[c2].to_ulong();
+	auto mask=a1234[c2].raw();
 
 	Matrices->states(c2+1) = allowed_states_for_mask[mask];
     }
@@ -95,25 +96,40 @@ sample_two_nodes_base(mutable_data_partition P, const data_partition& P0, const
 
     Matrices->forward();
 
+    //------------- Sample a path from the matrix -------------------//
+
+    const auto& nodes = order.nodes;
+
+    int b04 = P.t().find_branch(nodes[0],nodes[4]);
+    int b14 = P.t().find_branch(nodes[1],nodes[4]);
+    int b25 = P.t().find_branch(nodes[2],nodes[5]);
+    int b35 = P.t().find_branch(nodes[3],nodes[5]);
+    int b45 = P.t().find_branch(nodes[4],nodes[5]);
+
     // If the DP matrix ended up having probability 0, don't try to sample a path through it!
     if (Matrices->Pr_sum_all_paths() <= 0.0) 
     {
 	std::cerr<<"sample_two_nodes_base( ): All paths have probability 0!"<<std::endl;
-	return Matrices;
+
+        // Computing likelihoods when pairwise alignments disagree about internal sequence
+        // lengths can yield memory access errors.
+        P.unset_pairwise_alignment(b04);
+        P.unset_pairwise_alignment(b14);
+        P.unset_pairwise_alignment(b25);
+        P.unset_pairwise_alignment(b35);
+        P.unset_pairwise_alignment(b45);
     }
+    else
+    {
+        vector<int> path_g = Matrices->sample_path();
+        vector<int> path = Matrices->ungeneralize(path_g);
 
-    //------------- Sample a path from the matrix -------------------//
-
-    vector<int> path_g = Matrices->sample_path();
-    vector<int> path = Matrices->ungeneralize(path_g);
-
-    const auto& nodes = order.nodes;
-
-    P.set_pairwise_alignment(P.t().find_branch(nodes[0],nodes[4]), get_pairwise_alignment_from_path(path, *Matrices, 0, 4));
-    P.set_pairwise_alignment(P.t().find_branch(nodes[1],nodes[4]), get_pairwise_alignment_from_path(path, *Matrices, 1, 4));
-    P.set_pairwise_alignment(P.t().find_branch(nodes[2],nodes[5]), get_pairwise_alignment_from_path(path, *Matrices, 2, 5));
-    P.set_pairwise_alignment(P.t().find_branch(nodes[3],nodes[5]), get_pairwise_alignment_from_path(path, *Matrices, 3, 5));
-    P.set_pairwise_alignment(P.t().find_branch(nodes[4],nodes[5]), get_pairwise_alignment_from_path(path, *Matrices, 4, 5));
+        P.set_pairwise_alignment(b04, get_pairwise_alignment_from_path(path, *Matrices, 0, 4));
+        P.set_pairwise_alignment(b14, get_pairwise_alignment_from_path(path, *Matrices, 1, 4));
+        P.set_pairwise_alignment(b25, get_pairwise_alignment_from_path(path, *Matrices, 2, 5));
+        P.set_pairwise_alignment(b35, get_pairwise_alignment_from_path(path, *Matrices, 3, 5));
+        P.set_pairwise_alignment(b45, get_pairwise_alignment_from_path(path, *Matrices, 4, 5));
+    }
 
     return Matrices;
 }
@@ -129,23 +145,18 @@ int sample_two_nodes_multi(vector<Parameters>& p,const vector<A5::hmm_order>& or
     vector<A5::hmm_order> order = order_;
     vector<log_double_t> rho = rho_;
     assert(p.size() == order.size());
-  
-/*
-//------------ Check the alignment branch constraints ------------//
-for(int i=0;i<p.size();i++) {
-vector<int> branches;
 
-branches.push_back(p[i].t().branch(order[i].nodes[0], order[i].nodes[4]));
-branches.push_back(p[i].t().branch(order[i].nodes[1], order[i].nodes[4]));
-branches.push_back(p[i].t().branch(order[i].nodes[2], order[i].nodes[5]));
-branches.push_back(p[i].t().branch(order[i].nodes[3], order[i].nodes[5]));
-branches.push_back(p[i].t().branch(order[i].nodes[4], order[i].nodes[5]));
+    double heat_beta = p[0].get_beta();
+    vector<optional<vector<HMM::bitmask_t>>> a123456(p[0].n_data_partitions());
 
-if (any_branches_constrained(branches, p[i].t(), p[i].PC->TC, p[i].PC->AC))
-return -1;
-}
-*/
+    for(int j=0;j<p[0].n_data_partitions();j++)
+        if (p[0][j].has_pairwise_alignments())
+            a123456[j] = A5::get_bitpath(p[0][j], order[0]);
   
+    vector< vector<log_double_t> > OS(p.size());
+    vector< vector<log_double_t> > OP(p.size());
+    vector<log_double_t> Pr(p.size());
+
     //----------- Generate the different states and Matrices ---------//
     log_double_t C1 = A5::correction(p[0],order[0]);
 #if !defined(NDEBUG_DP) || !defined(NDEBUG)
@@ -153,31 +164,28 @@ return -1;
 #endif
 
     vector< vector< shared_ptr<DParrayConstrained> > > Matrices(p.size());
-    for(int i=0;i<p.size();i++) 
-	for(int j=0;j<p[i].n_data_partitions();j++) 
+    for(int i=0;i<p.size();i++)
+    {
+	for(int j=0;j<p[i].n_data_partitions();j++)
+        {
 	    if (p[i][j].variable_alignment())
 	    {
-		Matrices[i].push_back(sample_two_nodes_base(p[i][j], p[0][j], order[i], order[0]));
-		//    p[i][j].LC.invalidate_node(p[i].T,order[i][4]);
-		//    p[i][j].LC.invalidate_node(p[i].T,order[i][5]);
+		Matrices[i].push_back(sample_two_nodes_base(p[i][j], *a123456[j], order[i], order[0]));
 		if (Matrices[i].back()->Pr_sum_all_paths() <= 0.0)
+                {
 		    std::cerr<<"Pr = 0   i = "<<i<<"   j="<<j<<" \n";
+                    // Avoid calculating likelihoods and priors if the alignment is unset.
+                    return -1;
+                }
 #ifndef NDEBUG
 		p[i][j].likelihood();  // check the likelihood calculation
 #endif
 	    }
 	    else
 		Matrices[i].push_back(NULL);
-	
+        }
 
-
-    //-------- Calculate corrections to path probabilities ---------//
-
-    vector< vector<log_double_t> > OS(p.size());
-    vector< vector<log_double_t> > OP(p.size());
-
-    for(int i=0; i<p.size(); i++) 
-    {
+        //-------- Calculate corrections to path probabilities ---------//
 	if (do_OS)
 	    for(int j=0;j<p[i].n_data_partitions();j++)
 		OS[i].push_back( p[i][j].likelihood() );
@@ -189,19 +197,15 @@ return -1;
 		OP[i].push_back( other_prior(p[i][j],order[i].nodes) );
 	else
 	    OP[i] = vector<log_double_t>(p[i].n_data_partitions(),log_double_t(1));
-    }
 
-    //---------------- Calculate choice probabilities --------------//
-    vector<log_double_t> Pr(p.size());
 
-    for(int i=0;i<Pr.size();i++) 
-    {
-	Pr[i] = rho[i] * p[i].prior_no_alignment();
+        //---------------- Calculate choice probabilities --------------//
+        Pr[i] = rho[i] * p[i].prior_no_alignment();
 
 	// sum of substitution and alignment probability over all paths
 	for(int j=0;j<p[i].n_data_partitions();j++)
 	{
-	    Pr[i] *= pow(OS[i][j], p[i][j].get_beta());
+	    Pr[i] *= pow(OS[i][j], heat_beta);
 	    if (p[i][j].variable_alignment())
 	    {
 		Pr[i] *= Matrices[i][j]->Pr_sum_all_paths();
