@@ -20,16 +20,24 @@
 #include <iostream>
 #include <fstream>
 #include <string>
+#include <optional>
 #include <regex>
 #include "alignment/alignment.H"
 #include "alignment/alignment-util.H"
-#include "util.H"
+#include "util/set.H"
+#include "util/mapping.H"
+#include "util/string/split.H"
+#include "util/string/convert.H"
+#include "util/io.H"
+#include "util/range.H"
+#include "util/cmdline.H"
 #include "sequence/sequence.H"
 #include "sequence/sequence-format.H"
 #include <boost/program_options.hpp>
-#include "io.H"
 #include "findroot.H"
 
+extern int log_verbose;
+
 using namespace sequence_format;
 namespace po = boost::program_options;
 using po::variables_map;
@@ -39,6 +47,7 @@ using std::istream;
 using std::vector;
 using std::string;
 using std::pair;
+using std::optional;
 
 using std::cin;
 using std::cout;
@@ -223,20 +232,6 @@ variables_map parse_cmd_line(int argc,char* argv[])
     return args;
 }
 
-void pad_to_same_length(vector<sequence>& s)
-{
-    // find total alignment length
-    vector<unsigned> L;
-    for(int i=0;i<s.size();i++)
-	L.push_back(s[i].size());
-    unsigned AL = max(L);
-
-    // pad sequences if they are less than this length
-    for(int i=0;i<s.size();i++)
-	if (L[i] < AL)
-	    (string&)s[i] = (string&)s[i] + string(AL-L[i],'-');
-}
-
 vector<sequence> load_file(istream& file,bool pad)
 {
     vector<sequence> s = sequence_format::read_guess(file);
@@ -466,7 +461,7 @@ vector<pair<int,int>> read_pairs(const string& filename, int first = 0)
 	{
 	    if (not pair_string.empty())
 	    {
-		auto pieces = split<int>(pair_string,':');
+		auto pieces = convertTo<int>(split(pair_string,':'));
 		if (pieces.size() != 2)
 		    throw myexception()<<"Malformed pair '"<<pair_string<<"'";
 		pairs.push_back({pieces[0] - first,pieces[1] - first});
@@ -485,21 +480,22 @@ int main(int argc,char* argv[])
 
 	bool pad = (args.count("pad")>0);
 
-	vector<string> names;
+	optional<vector<string>> names;
 	if (args.count("taxa"))
-	    names = split(args["taxa"].as<string>(),',');
-	else if (args.count("reorder-by-tree"))
+	    names = get_string_list(args, "taxa");
+	else if (auto tree = get_arg<string>(args,"reorder-by-tree"))
 	{
 	    RootedSequenceTree RT;
-	    RT.read(args["reorder-by-tree"].as<string>());
+	    RT.read(*tree);
 	    bool use_root = args.count("use-root");
 	    names = get_names_from_tree(RT, use_root);
 	}
-	else if (args.count("reorder-by-alignment"))
+	else if (auto align = get_arg<string>(args, "reorder-by-alignment"))
 	{
-	    vector<sequence> sequences = load_file(args["reorder-by-alignment"].as<string>(), false);
+	    vector<sequence> sequences = load_file(*align, false);
+            names = vector<string>();
 	    for(const auto& s: sequences)
-		names.push_back(s.name);
+		names->push_back(s.name);
 	}
 
 	//------- Determine filenames --------//
@@ -533,8 +529,8 @@ int main(int argc,char* argv[])
 		if (filenames.size() > 1)
 		    check_all_same_length(s, "in order to concatenate two or more alignments.");
 	
-		if (names.size())
-		    s = select_taxa(s,names);
+		if (names)
+		    s = select_taxa(s,*names);
 		S = concatenate(S,s);
 	    }
 	    catch (std::exception& e) {
