@@ -22,20 +22,23 @@
 #include <cmath>
 #include <vector>
 #include <list>
-#include "myexception.H"
+#include "util/myexception.H"
 #include "alignment/alignment.H"
-#include "tree/tree-util.H"
-#include "optimize.H"
-#include "findroot.H"
-#include "util.H"
-#include "setup.H"
 #include "alignment/alignment-util.H"
 #include "alignment/load.H"
+#include "tree/tree-util.H"
+#include "tree-align/link.H"
+#include "optimize.H"
+#include "findroot.H"
+#include "util/mapping.H"
+#include "util/range.H"
 #include "distance-methods.H"
 
 #include <boost/program_options.hpp>
 #include <boost/dynamic_bitset.hpp>
 
+extern int log_verbose;
+
 namespace po = boost::program_options;
 using std::vector;
 using std::string;
@@ -292,7 +295,7 @@ double get_column_probability(const vector<int>& column,
 {
     unsigned int count=0;
     int i=0;
-    foreach(A,alignments) {
+    for(const auto& A: alignments) {
 	bool found=true;
 
 	// Can we find a common column for all features?
@@ -324,7 +327,7 @@ double get_column_probability(const vector<int>& column,
 		if (column[j] != alphabet::gap) continue;
 	
 		// if the template doesn't have a gap, then this doesn't match
-		if (A->character(c,j))
+		if (A.character(c,j))
 		    found = false;
 	    }
 	}
@@ -350,7 +353,7 @@ variables_map parse_cmd_line(int argc,char* argv[])
 	("alphabet",value<string>(),"set to 'Codons' to prefer codon alphabets")
 	("skip",value<unsigned>()->default_value(0),"number of alignment samples to skip")
 	("max-alignments",value<int>()->default_value(1000),"maximum number of alignments to analyze")
-	("verbose,v","Output more log messages on stderr.")
+	("verbose,V","Output more log messages on stderr.")
 	;
 
     // positional options
@@ -527,8 +530,8 @@ int main(int argc,char* argv[]) {
 	list<alignment> alignments;
 	vector<matrix<int> > Ms;
 	do_setup(args,alignments,A,RT);
-	foreach(i,alignments)
-	    Ms.push_back(M(*i));
+	for(const auto& i: alignments)
+	    Ms.push_back(M(i));
 
 	SequenceTree T = RT;
 	remove_sub_branches(T);
@@ -545,8 +548,8 @@ int main(int argc,char* argv[]) {
 
 	//----------- Construct alignment indexes ----------//
 	vector< vector< vector<int> > >  column_indexes;
-	foreach(i,alignments)
-	    column_indexes.push_back( column_lookup(*i,T.n_leaves()) );
+	for(const auto& i: alignments)
+	    column_indexes.push_back( column_lookup(i,T.n_leaves()) );
 
 	//------- Convert template to index form-------//
 	matrix<int> MA = M(A);
