@@ -25,12 +25,17 @@
 #include "alignment/alignment-util.H"
 #include "alignment/load.H"
 #include "tree/tree-util.H"
-#include "util.H"
-#include "setup.H"
+#include "util/mapping.H"
+#include "util/range.H"
+#include "util/cmdline.H"
 #include "distance-methods.H"
 #include <utility>
 #include "alignment/index-matrix.H"
 
+#include <boost/dynamic_bitset.hpp>
+
+extern int log_verbose;
+
 using std::pair;
 using std::vector;
 using std::string;
@@ -46,6 +51,7 @@ using std::cerr;
 using std::endl;
 
 using std::string;
+using boost::dynamic_bitset;
 
 variables_map parse_cmd_line(int argc,char* argv[]) 
 { 
@@ -55,7 +61,7 @@ variables_map parse_cmd_line(int argc,char* argv[])
     options_description general("General options");
     general.add_options()
 	("help,h", "Print usage information.")
-	("verbose,v","Output more log messages on stderr.");
+	("verbose,V","Output more log messages on stderr.");
 
     options_description invisible("Invisible options");
     invisible.add_options()
@@ -63,26 +69,31 @@ variables_map parse_cmd_line(int argc,char* argv[])
 
     options_description seq_filter("Sequence filtering options");
     seq_filter.add_options()
-	("protect",value<string>(),"Sequences that cannot be removed (comma-separated).")
-	("remove",value<string>(),"Remove sequences in comma-separated list <arg>.")
-	("longer-than",value<unsigned>(),"Remove sequences not longer than <arg>.")
-	("shorter-than",value<unsigned>(),"Remove sequences not shorter than <arg>.")
-	("cutoff",value<unsigned>(),"Remove similar sequences with #mismatches < cutoff.")
-	("down-to",value<int>(),"Remove similar sequences down to <arg> sequences.")
+	("protect,p",value<string>(),"Sequences that cannot be removed (comma-separated).")
+	("keep,k",value<string>(),"Remove sequences not in comma-separated list <arg>.")
+	("remove,r",value<string>(),"Remove sequences in comma-separated list <arg>.")
+	("longer-than,l",value<unsigned>(),"Remove sequences not longer than <arg>.")
+	("shorter-than,s",value<unsigned>(),"Remove sequences not shorter than <arg>.")
+	("cutoff,c",value<unsigned>(),"Remove similar sequences with #mismatches < cutoff.")
+	("down-to,d",value<int>(),"Remove similar sequences down to <arg> sequences.")
 	("remove-crazy",value<int>(),"Remove <arg> outlier sequences -- defined as sequences that are missing too many conserved sites.")
 	("conserved",value<double>()->default_value(0.75),"Fraction of sequences that must contain a letter for it to be considered conserved.")
 	;
 
     options_description col_filter("Column filtering options");
     col_filter.add_options()
-	("min-letters",value<int>(),"Remove columns with fewer than <arg> letters.")
-	("remove-unique",value<int>(),"Remove insertions in a single sequence if longer than <arg> letters");
+        ("keep-columns,K",value<string>(),"Keep columns from this sequence")
+	("min-letters,m",value<int>(),"Remove columns with fewer than <arg> letters.")
+	("remove-unique,u",value<int>(),"Remove insertions in a single sequence if longer than <arg> letters")
+	("erase-empty-columns,e","Remove columns with no characters (all gaps).");
 
     options_description output("Output options");
     output.add_options()
-	("sort","Sort partially ordered columns to group similar gaps.")
-	("show-lengths","Just print out sequence lengths.")
-	("find-dups", value<string>(),"For each sequence, find the closest other sequence.");
+	("sort,S","Sort partially ordered columns to group similar gaps.")
+	("show-lengths,L","Just print out sequence lengths.")
+	("show-names,N","Just print out sequence lengths.")
+	("find-dups,F", value<string>(),"For each sequence, find the closest other sequence.")
+        ;
 
     // positional options
     positional_options_description p;
@@ -109,16 +120,18 @@ variables_map parse_cmd_line(int argc,char* argv[])
 	cout<<"   % alignment-thin --min-letters=5 file.fasta > file-thinned.fasta\n\n";
 	cout<<" Remove sequences by name:\n";
 	cout<<"   % alignment-thin --remove=seq1,seq2 file.fasta > file2.fasta\n\n";
+	cout<<"   % alignment-thin --keep=seq1,seq2   file.fasta > file2.fasta\n\n";
 	cout<<" Remove short sequences:\n";
 	cout<<"   % alignment-thin --longer-than=250 file.fasta > file-long.fasta\n\n";
-	cout<<" Remove sequences with <= 5 differences from the closest other sequence:\n";
+	cout<<" Remove similar sequences with <= 5 differences from the closest other sequence:\n";
 	cout<<"   % alignment-thin --cutoff=5 file.fasta > more-than-5-differences.fasta\n\n";
-	cout<<" Like --cutoff, but stop when we have the right number of sequences:\n";
+	cout<<" Remove similar sequences until we have the right number of sequences:\n";
 	cout<<"   % alignment-thin --down-to=30 file.fasta > file-30taxa.fasta\n\n";
 	cout<<" Remove dissimilar sequences that are missing conserved columns:\n";
 	cout<<"   % alignment-thin --remove-crazy=10 file.fasta > file2.fasta\n\n";
 	cout<<" Protect some sequences from being removed:\n";
 	cout<<"   % alignment-thin --down-to=30 file.fasta --protect=seq1,seq2 > file2.fasta\n\n";
+	cout<<"   % alignment-thin --down-to=30 file.fasta --protect=@filename > file2.fasta\n\n";
 
 	exit(0);
     }
@@ -268,24 +281,14 @@ matrix<int> symmetric_overlap_matrix(const alignment& A)
     return D;
 }
 
-int remove_almost_empty_columns(alignment& A,int n)
+dynamic_bitset<> enough_letters(alignment& A, int n)
 {
-    int length = 0;
+    dynamic_bitset<> keep(A.length());
 
     for(int column=0;column<A.length();column++)
-	if (n_characters(A,column) >= n) 
-	{
-	    if (column != length)
-		for(int i=0;i<A.n_sequences();i++)
-		    A.set_value(length,i, A(column,i) );
-	    length++;
-	}
-
-    int n_empty = A.length() - length;
-
-    A.changelength(length);
-
-    return n_empty;
+	if (n_characters(A,column) >= n)
+            keep[column] = true;
+    return keep;
 }
 
 int unique_letter(const alignment& A,int c)
@@ -312,14 +315,64 @@ vector<int> get_taxon_indices(const vector<string>& names, const vector<string>&
     return indices;
 }
 
-vector<int> get_taxon_indices(const vector<string>& names,const string& lookup)
+dynamic_bitset<> column_contains_sequence(const alignment& A, int index)
 {
-    vector<string> lookup_v = split(lookup,',');
-
-    return get_taxon_indices(names,lookup_v);
+    dynamic_bitset<> present(A.length());
+    for(int c=0; c < A.length(); c++)
+        present[c] = A.character(c,index)?1:0;
+    return present;
 }
 
 
+dynamic_bitset<> part_of_long_insertion(const alignment& A, int L)
+{
+    dynamic_bitset<> unique(A.length());
+    for(int i=0;i<A.length();i++)
+        unique[i] = unique_letter(A,i);
+
+    vector<vector<int> > columns = column_lookup(A);
+
+    dynamic_bitset<> in_long_insertion(A.length());
+
+    for(int i=0;i<A.n_sequences();i++)
+    {
+        int count = 0;
+
+        for(int j=0;j<columns[i].size()+1;j++)
+        {
+            int u = -1;
+            if (j< columns[i].size())
+                u = unique[columns[i][j]];
+
+            if (u == i)
+                count++;
+            else {
+                // close out the current insertion if there is one
+                if (count > L) 
+                {
+                    // remove count-L sites from the middle of the insertion
+                    int start = j - count;
+                    int end   = j - 1;
+
+                    if (start == 0)
+                        end -= L;
+                    else if (end == columns[i].size()-1)
+                        start += L;
+                    else {
+                        start += L/2;
+                        end   -= (L-L/2);
+                    }
+
+                    for(int k=start;k<=end;k++)
+                        in_long_insertion[columns[i][k]] = 1;
+                }
+                count = 0;
+            }
+        }
+    }
+    return in_long_insertion;
+}
+
 int main(int argc,char* argv[])
 {
     try {
@@ -330,7 +383,7 @@ int main(int argc,char* argv[])
 	variables_map args = parse_cmd_line(argc,argv);
 
 	//----------- Load alignment and tree ---------//
-	alignment A = load_A(args,false);
+	alignment A = load_A(args,true,false);
 	const int N = A.n_sequences();
 	const int L = A.length();
 
@@ -343,9 +396,19 @@ int main(int argc,char* argv[])
 	for(int i=0;i<N;i++)
 	    AL[i] = A.seqlength(i);
 
-	if (args.count("show-lengths")) {
-	    for(int i=0;i<A.n_sequences();i++) {
-		cout<<AL[i]<<endl;
+	if (args.count("show-lengths") or args.count("show-names"))
+	{
+	    bool show_lengths = args.count("show-lengths");
+	    bool show_names = args.count("show-names");
+	    for(int i=0;i<A.n_sequences();i++)
+	    {
+		if (show_names)
+		    cout<<names[i];
+		if (show_names and show_lengths)
+		    cout<<",";
+		if (show_lengths)
+		    cout<<AL[i];
+		cout<<endl;
 	    }
 	    exit(0);
 	}
@@ -355,9 +418,7 @@ int main(int argc,char* argv[])
 	// By default every sequence has status 1 which means, removeable, but not removed.
 	vector<int> keep(A.n_sequences(),1);
 
-	vector<string> protect;
-	if (args.count("protect"))
-	    protect = split(args["protect"].as<string>(),',');
+	vector<string> protect = get_string_list(args, "protect");
 
 	for(int i=0;i<protect.size();i++)
 	{
@@ -389,11 +450,27 @@ int main(int argc,char* argv[])
 
 	//-------------------- remove ------------------------//
 
-	vector<string> remove;
-	if (args.count("remove"))
-	    remove = split(args["remove"].as<string>(),',');
+	if (args.count("keep"))
+	{
+	    if (args.count("remove")) throw myexception()<<"You cannot specify both 'keep' and 'remove'!";
 
-	for(auto& r: remove)
+	    // FIXME: Currently we protect these and remove everything else.
+	    //        Should instead do remove-all-except these?
+	    for(auto& r: get_string_list(args, "keep"))
+	    {
+		if (auto r_index = find_index(names,r))
+		{
+		    keep[*r_index] = 2;
+		}
+		else
+		    throw myexception()<<"remove: can't find sequence '"<<r<<"' to remove.";
+	    }
+	    for(auto& k: keep)
+		if (k != 2)
+		    k=0;
+	}
+
+	for(auto& r: get_string_list(args, "remove"))
 	{
 	    if (auto r_index = find_index(names,r))
 	    {
@@ -472,7 +549,7 @@ int main(int argc,char* argv[])
 	
       
 	    // get the indices for the taxa to compare to
-	    vector<int> compare_to = get_taxon_indices(names, args["find-dups"].as<string>());
+	    vector<int> compare_to = get_taxon_indices(names, parse_string_list(args["find-dups"].as<string>()));
 
 	    // convert the indices to a mask
 	    vector<int> target(names.size(),0);
@@ -587,74 +664,39 @@ int main(int argc,char* argv[])
 	//------- Select the sequences -------//
 	alignment A2 = select_rows(A,keep);
 
+        dynamic_bitset<> site_is_protected(A2.length());
+        if (args.count("keep-columns"))
+        {
+            string reference = args["keep-columns"].as<string>();
+            auto index = find_index(names,reference);
+            if (not index)
+                throw myexception()<<"--keep-columns: Can't find sequence '"<<reference<<"'";
+            site_is_protected = column_contains_sequence(A, *index);
+        }
+
 	//------- Remove consecutive unique sites -----//
+        dynamic_bitset<> keep_sites(A2.length());
+        keep_sites.flip();
 
-	// Question: should I remove ALL of the sites, or just the sites beyond L aa/nucs?
-	//    And, how would I remove the MIDDLE sites?
-	// Question: should I remove only sites that have no ?  How should I do so?
 	if (args.count("remove-unique"))
 	{
+            // Question: should I remove ALL of the sites, or just the sites beyond L aa/nucs?
+            //    And, how would I remove the MIDDLE sites?
+            // Question: should I remove only sites that have no ?  How should I do so?
+
 	    int L = args["remove-unique"].as<int>();
-
-	    vector<int> keep_sites(A2.length(),1);
-
-	    vector<int> unique(A2.length(),-1);
-	    for(int i=0;i<A2.length();i++)
-		unique[i] = unique_letter(A,i);
-
-	    vector<vector<int> > columns = column_lookup(A);
-
-	    for(int i=0;i<A.n_sequences();i++)
-	    {
-		int count =0;
-
-		for(int j=0;j<columns[i].size()+1;j++)
-		{
-		    int u = -1;
-		    if (j< columns[i].size())
-			u = unique[columns[i][j]];
-
-		    if (u == i)
-			count++;
-		    else {
-			// close out the current insertion if there is one
-			if (count > L) 
-			{
-			    // remove count-L sites from the middle of the insertion
-			    int start = j - count;
-			    int end   = j - 1;
-
-			    if (start == 0)
-				end -= L;
-			    else if (end == columns[i].size()-1)
-				start += L;
-			    else {
-				start += L/2;
-				end   -= (L-L/2);
-			    }
-
-			    for(int k=start;k<=end;k++)
-				keep_sites[columns[i][k]]=0;
-			}
-			count = 0;
-		    }
-		}
-	    }
-
-	    // actually remove the unwanted sites
-	    vector<int> sites;
-	    for(int i=0;i<keep_sites.size();i++)
-		if (keep_sites[i])
-		    sites.push_back(i);
-	    A2 = select_columns(A2,sites);
+            keep_sites &= ~part_of_long_insertion(A2, L);
 	}
 
+	// ------- Remove columns with too few letters ------ //
+	if (args.count("min-letters"))
+            keep_sites &= enough_letters( A2, args["min-letters"].as<int>() );
 
-	//------- Remove columns with too few letters ------//
-	if (args.count("min-letters")) {
-	    int m = args["min-letters"].as<int>();
-	    remove_almost_empty_columns(A2,m);
-	}
+        // ------- Actually remove columns ----------------- //
+        A2 = select_columns(A2, keep_sites | site_is_protected);
+
+	if (args.count("erase-empty-columns")) 
+	    remove_empty_columns(A2);
 
 	if (args.count("sort"))
 	    A2 = get_ordered_alignment(A2);
