@@ -18,7 +18,12 @@
   <http://www.gnu.org/licenses/>.  */
 
 #include "mctree.H"
-#include "rng.H"
+#include "util/rng.H"
+#include "util/range.H"
+#include "util/string/join.H" 
+#include "util/string/convert.H"
+
+extern int log_verbose;
 
 using std::vector;
 using std::string;
