@@ -45,7 +45,13 @@
 #include <list>
 
 #include "tree/sequencetree.H"
-#include "util.H"
+#include "util/string/join.H"
+#include "util/string/convert.H"
+#include "util/set.H"
+#include "util/range.H"
+#include "util/mapping.H"
+#include "util/log-level.H"
+#include "util/index_value.H"
 #include "statistics.H"
 #include "bootstrap.H"
 #include "tree-dist.H"
@@ -53,6 +59,8 @@
 
 #include <boost/program_options.hpp>
 
+extern int log_Verbose;
+
 namespace po = boost::program_options;
 using po::variables_map;
 
@@ -67,7 +75,7 @@ using std::valarray;
 using std::ios;
 
 using boost::dynamic_bitset;
-using boost::optional;
+using std::optional;
 
 using namespace statistics;
 
@@ -490,7 +498,7 @@ variables_map parse_cmd_line(int argc,char* argv[])
 	("subsample,x",value<int>(),"Factor by which to sub-sample.")
 	("predicates",value<string>(),"Predicates to examine.")
 	("min-support",value<double>()->default_value(0.1,"0.1"),"Minimum value of predicates to consider interesting..")
-	("verbose,v","Output more log messages on stderr.")
+	("verbose,V","Output more log messages on stderr.")
 	;
   
     options_description bootstrap("Block bootstrap options");
@@ -520,9 +528,7 @@ variables_map parse_cmd_line(int argc,char* argv[])
     p.add("files", -1);
   
     variables_map args;     
-    store(command_line_parser(argc, argv).
-	  options(all).positional(p).run(), args);
-    // store(parse_command_line(argc, argv, desc), args);
+    store(command_line_parser(argc, argv).options(all).positional(p).run(), args);
     notify(args);    
 
     if (args.count("help")) {
@@ -579,11 +585,11 @@ am_sdsf(const tree_sample_collection& tree_dists, int d, const map<dynamic_bitse
     int N = tree_dists.n_samples(d);
   
     // For each sampled partition
-    foreach(record,counts) 
+    for(const auto& record: counts)
     {
 	// Skip records that have a frequency that is too low
 	bool skip=true;
-	const vector<int>& x = record->second.counts;
+	const vector<int>& x = record.second.counts;
 	for(int i=0;i<N;i++) {
 	    if (x[tree_dists.index(d,i)] > min[i]) {
 		skip = false;
@@ -760,13 +766,13 @@ int main(int argc,char* argv[])
 
 	    // For each sampled partition
 	    int n_splits=0;
-	    foreach(record,counts) 
+	    for(const auto& record: counts)
 	    {
 		n_splits++;
 		// Skip records in which all frequencies are too low
 		bool skip=true;
 		int count = 0;
-		const vector<int>& x = record->second.counts;
+		const vector<int>& x = record.second.counts;
 		for(int i=0;i<x.size();i++) {
 		    count += x[i];
 		    if (x[i] > min[i])
@@ -775,7 +781,7 @@ int main(int argc,char* argv[])
     
 		// Add this standard deviation to the sum
 		if (not skip ) {
-		    splits.push_back(record->first);
+		    splits.push_back(record.first);
 		    counts2.push_back(count);
 		}
 	    }
@@ -914,7 +920,7 @@ int main(int argc,char* argv[])
 		if (VS[d][T][p].nchanges_ave > 10)
 		    worst_Ne.check_min(p,VS[d][T][p].Ne);
 	    }
-	    cout<<"min Ne = "<<worst_Ne.value<<"    (partition = "<<worst_Ne.index+1<<")"<<endl;
+	    cout<<"min Ne = "<<worst_Ne.value<<"    (partition = "<<*worst_Ne.index+1<<")"<<endl;
 	}
     }
     catch (std::exception& e) {
