@@ -32,16 +32,21 @@
 #include <list>
 
 #include "tree/sequencetree.H"
-#include "util.H"
+#include "util/string/split.H"
+#include "util/cmdline.H"
+#include "util/string/convert.H"
+#include "util/set.H"
+#include "util/range.H"
 #include "statistics.H"
 #include "bootstrap.H"
 #include "tree-dist.H"
 #include "consensus-tree.H"
 #include "mctree.H"
-#include "rng.H"
+#include "util/rng.H"
 
 #include <boost/program_options.hpp>
-#include <boost/optional.hpp>
+
+extern int log_verbose;
 
 /// FIXME - construct a full tree by extending the moveable
 ///  a) randomly
@@ -79,7 +84,7 @@ using std::ios;
 using std::ofstream;
 
 using statistics::odds;
-using boost::optional;
+using std::optional;
 
 // What if everything in 'split' is true?
 // What if everything in 'split' is true, but 1 taxa?
@@ -110,21 +115,16 @@ struct consensus_level_order
 };
 
 // FIXME - if n is small, first level may be ~ 0.57 -- good or bad?
-vector<pair<double, string> > get_consensus_levels(const string& s) 
+vector<pair<double, string> > get_consensus_levels(const vector<string>& levels)
 {
-    // split into comma-separate components
-    vector<string> levels;
-    if (s.size())
-	levels = split(s,',');
-
     vector<pair<double, string> > parsed_levels;
 
     // Split each component level>file into level (a double) and file (a string)
-    for(int i=0;i<levels.size();i++) 
+    for(auto& level: levels)
     {
-	if (not levels[i].size()) continue;
-	vector<string> parts = split(levels[i],':');
-	if (parts.size() > 2) throw myexception()<<"Consensus level '"<<levels[i]<<"' has too many files.";
+	if (not level.size()) continue;
+	vector<string> parts = split(level,':');
+	if (parts.size() > 2) throw myexception()<<"Consensus level '"<<level<<"' has too many files.";
 	if (parts.size() == 1) parts.push_back("-");
 
 	assert(parts.size() == 2);
@@ -401,28 +401,26 @@ variables_map parse_cmd_line(int argc,char* argv[])
     options_description input("Input options");
     input.add_options()
 	("help,h", "Produce help message.")
+	("verbose,V","Output more log messages on stderr.")
 	("skip,s",value<string>()->default_value("10%"),"Number of trees to skip.")
 	("until,u",value<int>(),"Read until this number of trees.")
 	("max,m",value<int>(),"Thin tree samples down to this number of trees.")
 	("subsample,x",value<int>()->default_value(1),"Factor by which to subsample.")
-	("ignore", value<string>(),"Comma-separated list of taxa to ignore.")
+	("ignore,i", value<string>(),"Comma-separated list of taxa to ignore.")
 	;
   
     options_description reporting("Reporting options");
     reporting.add_options()
 	("map-trees",value<int>()->default_value(1),"Only report the top <arg> trees per file.")
-	("map-tree",value<string>(),"Write out the map tree to file <arg>.")
 	("min-support",value<double>()->default_value(0.25),"Minimum threshold PP for splits.")
 	("report",value<string>(),"Write supported partitions to file <arg>.")
-	("consensus-PP",value<string>(),"Write out consensus trees+PP.")
+	("map-tree",value<string>(),"Write out the map tree to file <arg>.")
 	("consensus",value<string>(),"Write out consensus trees.")
 	("greedy-consensus",value<string>(),"Write out greedy consensus trees.")
 	("extended-consensus-L",value<string>(),"Write out extended consensus trees + lengths.")
 	("extended-consensus",value<string>(),"Write out extended consensus trees.")
 	("support-levels",value<string>(),"Write #branches versus LOD to file <arg>.")
 	("extended-support-levels",value<string>(),"Write #sub-branches versus LOD to file <arg>.")
-	("odds-ratio",value<double>()->default_value(1.5),"Report partial-splits only if removing taxa improves the odds by at least this ratio.")
-	("verbose,v","Output more log messages on stderr.")
 	;
     
     options_description search("Search options");
@@ -430,6 +428,7 @@ variables_map parse_cmd_line(int argc,char* argv[])
 	("sub-partitions","Search for partial splits.")
 	("depth",value<int>()->default_value(1),"Depth at which to look for partial splits.")
 	("rooting",value<double>()->default_value(0.9,"0.9"),"Threshold in search for partial splits.")
+	("odds-ratio",value<double>()->default_value(1.5),"Report partial-splits only if removing taxa improves the odds by at least this ratio.")
 	;
 
     options_description visible("All options");
@@ -452,9 +451,21 @@ variables_map parse_cmd_line(int argc,char* argv[])
 	cout<<input<<"\n";
 	cout<<reporting<<"\n";
 	cout<<search<<"\n";
-	cout<<"Arguments for consensus trees are level1:filename1[,level2:filename2,...]\n\
-  o each level is a minimum PP and should be in the range [0.5, 1.0].\n\
-  o drop \"filename\" or specify \":-\" to write to the terminal.\n";
+	cout<<"Examples:\n\n";
+	cout<<" Compute the majority consensus tree, skipping the first 10% of trees:\n";
+	cout<<"   % trees-consensus newick.trees > c50.tree\n";
+	cout<<"   % trees-consensus newick.trees --skip=10% > c50.tree\n\n";
+	cout<<" Skip the first 100 trees:\n";
+	cout<<"   % trees-consensus newick.trees --skip=100 > c50.tree\n\n";
+	cout<<" Skip the first 20% of trees and take every 10th tree thereafter:\n";
+	cout<<"   % trees-consensus newick.trees --skip=20% -x10 > c50.tree\n\n";
+	cout<<" Compute the 50% (majority) and 80% consensus trees:\n";
+	cout<<"   % trees-consensus newick.trees --consensus=0.5:c50.tree,0.8:c80.tree\n";
+	cout<<"   % trees-consensus newick.trees --consensus=0.5,0.8:c80.tree > c50.tree\n\n";
+	cout<<" Compute the MAP tree and write a summary of supported partitions:\n";
+	cout<<"   % trees-consensus --map-tree=MAP.tree --report=partitions.txt\n\n";
+	cout<<" Compute the MAP tree and write it to the standard output:\n";
+	cout<<"   % trees-consensus --map-tree=- --report=partitions.txt\n\n";
 	exit(0);
     }
 
@@ -479,14 +490,14 @@ struct count_more {
 /// \param N The total number of sampled trees
 void write_support_level_graph(variables_map& args, const vector<pair<partition,unsigned> >& all_partitions, unsigned N)
 {
-    if (!args.count("support-levels")) return;
-    const string filename = args["support-levels"].as<string>();
+    auto filename = get_arg<string>(args, "support-levels");
+    if (not filename) return;
 
     vector<unsigned> levels = get_Ml_levels(all_partitions, N, 0.5);
 
-    ofstream file(filename.c_str());
+    ofstream file(filename->c_str());
     if (!file)
-	throw myexception()<<"Couldn't open file '"<<filename<<"' for writing.";
+	throw myexception()<<"Couldn't open file '"<<*filename<<"' for writing.";
   
     for(int j=0;j<levels.size();j++) 
     {
@@ -595,15 +606,38 @@ void get_branch_lengths(SequenceTree& T,
     get_branch_lengths_and_PP(T, v, partitions, N);
 }
 
+void write_tree_with_PP(SequenceTree T,
+			const tree_sample& tree_dist,
+			const map<dynamic_bitset<>,count_and_length>& full_partitions,
+			const string& filename,
+			const string& description)
+{
+    int N = (int)tree_dist.size();
+
+    // set branch lengths and PP on the tree
+    vector<double> PP;
+    get_branch_lengths_and_PP(T, PP, full_partitions, N);
+  
+    bool show_branch_lengths = false;
+    for(int b=0;b<T.n_branches();b++)
+	if (T.branch(b).length() >= 0)
+	    show_branch_lengths = true;
+  
+    ostream_or_ofstream output(cout, "-", filename, description);
+
+    // write out the tree
+    output.unsetf(ios::fixed | ios::showpoint);
+    output<<T.write_with_bootstrap_fraction(PP,show_branch_lengths)<<std::endl;
+}
+
 /// \brief Write out standard majority consensus trees of various support levels
 ///
 /// \param tree_dist The sampled trees
 /// \param full_partitions The count and average lengths of each split
 /// \param consensus_levels The support levels and the output file name for each one
-/// \param with_PP Should the output trees contains Posterior Probabilities in addition to branch lengths?
 ///
 void write_consensus_trees(const tree_sample& tree_dist, const map<dynamic_bitset<>,count_and_length>& full_partitions, 
-			   const vector<pair<double, string> >& consensus_levels, bool with_PP)
+			   const vector<pair<double, string> >& consensus_levels)
 {
     unsigned N = tree_dist.size();
 
@@ -615,8 +649,6 @@ void write_consensus_trees(const tree_sample& tree_dist, const map<dynamic_bitse
     {
 	const string& filename = consensus_levels[k].second;
 
-	ostream_or_ofstream output(cout,"-",filename,"consensus tree file");
-
 	container_t partitions = select_splits(c50_partitions, N*consensus_levels[k].first);
 
 	// construct the consensus topology
@@ -629,21 +661,7 @@ void write_consensus_trees(const tree_sample& tree_dist, const map<dynamic_bitse
 	    }
 	}
 
-	// set branch lengths and PP on the consensus tree
-	vector<double> PP;
-	get_branch_lengths_and_PP(consensus, PP, partitions, N);
-
-	bool show_branch_lengths = false;
-	for(int b=0;b<consensus.n_branches();b++)
-	    if (consensus.branch(b).length() >= 0)
-		show_branch_lengths = true;
-
-	// write out the consensus tree
-	output.unsetf(ios::fixed | ios::showpoint);
-	if (with_PP)
-	    output<<consensus.write_with_bootstrap_fraction(PP,show_branch_lengths)<<std::endl;
-	else
-	    output<<consensus.write(show_branch_lengths)<<std::endl;
+	write_tree_with_PP(consensus, tree_dist, full_partitions, filename, "consensus tree file");
     }
 }
 
@@ -841,7 +859,6 @@ void output_mctree_with_lengths(ostream& output, MC_tree Q,
 /// \param tree_dist The sampled trees
 /// \param full_partitions The count and average lengths of each split
 /// \param consensus_levels The support levels and the output file name for each one
-/// \param with_PP Should the output trees contains Posterior Probabilities in addition to branch lengths?
 ///
 void write_extended_consensus_trees_with_lengths(const tree_sample& tree_dist, 
 						 const vector<pair<partition,unsigned> >& all_partitions,
@@ -908,10 +925,8 @@ bool compatible(const std::vector<dynamic_bitset<> >&P1, const dynamic_bitset<>&
 
 void write_greedy_consensus(const tree_sample& tree_dist,
 			    const map<dynamic_bitset<>,count_and_length>& full_partitions,
-			    const string& filename, bool with_PP)
+			    const string& filename)
 {
-    unsigned N = tree_dist.size();
-
     unsigned L = tree_dist.names().size();
   
     multiset< pair<dynamic_bitset<>,count_and_length>, count_more2 > sorted_splits;
@@ -934,22 +949,7 @@ void write_greedy_consensus(const tree_sample& tree_dist,
 	    consensus.induce_partition(S[i]);
 
     // set branch lengths and PP on the consensus tree
-    vector<double> PP;
-    get_branch_lengths_and_PP(consensus, PP, full_partitions, N);
-  
-    bool show_branch_lengths = false;
-    for(int b=0;b<consensus.n_branches();b++)
-	if (consensus.branch(b).length() >= 0)
-	    show_branch_lengths = true;
-  
-    ostream_or_ofstream output(cout,"-",filename,"greedy consensus tree file");
-
-    // write out the consensus tree
-    output.unsetf(ios::fixed | ios::showpoint);
-    if (with_PP)
-	output<<consensus.write_with_bootstrap_fraction(PP,show_branch_lengths)<<std::endl;
-    else
-	output<<consensus.write(show_branch_lengths)<<std::endl;
+    write_tree_with_PP(consensus, tree_dist, full_partitions, filename, "greedy consensus tree file");
 }
 
 int main(int argc,char* argv[]) 
@@ -977,22 +977,17 @@ int main(int argc,char* argv[])
 	bool show_sub = args.count("sub-partitions") or args.count("extended-consensus") or args.count("extended-consensus-L");
 
 	// consensus levels 
-	string c_levels = args.count("consensus") ? args["consensus"].as<string>() : "";
-	vector<pair<double,string> > consensus_levels = get_consensus_levels(c_levels);
-	string c_levels_pp = args.count("consensus-PP") ? args["consensus-PP"].as<string>() : "";
-	vector<pair<double,string> > consensus_levels_pp = get_consensus_levels(c_levels_pp);
-	string ec_levels = args.count("extended-consensus") ? args["extended-consensus"].as<string>() : "";
-	vector<pair<double,string> > extended_consensus_levels = get_consensus_levels(ec_levels);
-	string ecl_levels = args.count("extended-consensus-L") ? args["extended-consensus-L"].as<string>() : "";
-	vector<pair<double,string> > extended_consensus_L_levels = get_consensus_levels(ecl_levels);
-	string greedy_filename = args.count("greedy-consensus") ? args["greedy-consensus"].as<string>() : "";
+	vector<pair<double,string> > consensus_levels = get_consensus_levels( get_string_list(args, "consensus"));
+	vector<pair<double,string> > extended_consensus_levels = get_consensus_levels( get_string_list(args, "extended-consensus"));
+	vector<pair<double,string> > extended_consensus_L_levels = get_consensus_levels( get_string_list(args, "extended-consensus-L"));
+	auto greedy_filename = get_arg<string>(args, "greedy-consensus");
 
 	if (not args.count("consensus") 
-	    and not args.count("consensus-PP") 
 	    and not args.count("extended-consensus")
+	    and not args.count("map-tree")
 	    and not args.count("extended-consensus-L")
 	    and not args.count("greedy-consensus"))
-	    consensus_levels_pp.push_back(pair<double,string>(0.5,"-"));
+	    consensus_levels.push_back(pair<double,string>(0.5,"-"));
 
 	ostream_or_ofstream report_file;
 
@@ -1031,7 +1026,7 @@ int main(int argc,char* argv[])
 
 	for(int i=0;i<tree_dist.size();i++)
 	{
-	    std::map<tree_record,int>::iterator record = topologies_index.find(tree_dist[i]);
+	    auto record = topologies_index.find(tree_dist[i]);
 	    if (record == topologies_index.end())
 	    {
 		which_topology.push_back(i);
@@ -1079,20 +1074,6 @@ int main(int argc,char* argv[])
 	    report_file<<"\n";
 	}
 
-	// write out the map tree
-	if (args.count("map-tree")) 
-	{
-	    string filename = args["map-tree"].as<string>();
-
-	    ostream_or_ofstream output(cout,"-",filename,"MAP tree file");
-
-	    SequenceTree map_tree = tree_dist.T(which_topology[order[0]]);
-	    get_branch_lengths(map_tree, full_partitions, N);
-
-	    output<<map_tree<<endl;
-	}
-
-    
 	for(int i=0,n=0;i<topology_counts.size();i++) 
 	{
 	    n += topology_counts[i];
@@ -1134,7 +1115,6 @@ int main(int argc,char* argv[])
 	    report_file<<endl<<endl;
 	}
 
-
 	//----------- display M[l] consensus levels ----------//
 	write_support_level_graph(args, all_partitions, N);
 	write_extended_support_level_graph(args, all_partitions, tree_dist, N);
@@ -1142,10 +1122,20 @@ int main(int argc,char* argv[])
 	//----------- display M[l] consensus trees ----------//
 	std::cout.precision(4);
 
-	write_consensus_trees(tree_dist, full_partitions, consensus_levels,false);
-	write_consensus_trees(tree_dist, full_partitions, consensus_levels_pp,true);
-	if (args.count("greedy-consensus"))
-	    write_greedy_consensus(tree_dist, full_partitions, greedy_filename, true);
+	// write out the map tree
+	if (args.count("map-tree")) 
+	{
+	    string filename = args["map-tree"].as<string>();
+
+	    SequenceTree map_tree = tree_dist.T(which_topology[order[0]]);
+
+	    write_tree_with_PP(map_tree, tree_dist, full_partitions, filename, "MAP tree file");
+	}
+
+    
+	write_consensus_trees(tree_dist, full_partitions, consensus_levels);
+	if (greedy_filename)
+	    write_greedy_consensus(tree_dist, full_partitions, *greedy_filename);
 	write_extended_consensus_trees(tree_dist, all_partitions, extended_consensus_levels);
 	write_extended_consensus_trees_with_lengths(tree_dist, all_partitions, full_partitions, extended_consensus_L_levels);
     }
