@@ -20,10 +20,12 @@
 #include <iostream>
 #include <fstream>
 #include <cmath>
+#include <set>
 #include "tree/sequencetree.H"
-#include "myexception.H"
-#include "util.H"
-#include "io.H"
+#include "util/myexception.H"
+#include "util/mapping.H"
+#include "util/string/convert.H" // for convertToString( )
+#include "util/io.H"
 
 using namespace std;
 
@@ -105,7 +107,10 @@ void SequenceTree::read(std::istream& file) {
 
 string SequenceTree::write(bool print_lengths) const 
 {
-    RootedSequenceTree RT(*this,0);
+    int root = 0;
+    if (n_nodes() > 2)
+	root = directed_branch(0).target();
+    RootedSequenceTree RT(*this,root);
     return RT.write(print_lengths);
 }
 
@@ -556,6 +561,13 @@ RootedSequenceTree& RootedSequenceTree::operator=(const RootedSequenceTree& T)
     return *this;
 }
 
+RootedSequenceTree::RootedSequenceTree(const RootedSequenceTree& RT)
+    :Tree(),RootedTree(),SequenceTree()
+{
+    operator=(RT);
+}
+
+
 //FIXME - return mapping of leaf nodes?  Of all nodes?
 void standardize(SequenceTree& T) 
 {
@@ -588,3 +600,28 @@ RootedSequenceTree standardized(const string& t)
     return T;
 }
 
+/// \brief Re-index the leaves of tree \a T so that the labels have the same ordering as in \a names.
+///
+/// \param T The leaf-labelled tree.
+/// \param names The ordered leaf labels.
+///
+void remap_T_leaf_indices(SequenceTree& T,const vector<string>& names)
+{
+    assert(names.size() == T.n_leaves());
+    //----- Remap leaf indices for T onto A's leaf sequence indices -----//
+    try {
+	vector<int> mapping = compute_mapping(T.get_leaf_labels(), names);
+
+	T.standardize(mapping);
+    }
+    catch(const bad_mapping<string>& b)
+    {
+	bad_mapping<string> b2 = b;
+	b2.clear();
+	if (b2.from == 0)
+	    b2<<"Couldn't find leaf sequence \""<<b2.missing<<"\" in names.";
+	else
+	    b2<<"Sequence '"<<b2.missing<<"' not found in the tree.";
+	throw b2;
+    }
+}
